##   -*- autoconf -*-

## Check for ORACLE/OCI8

AC_DEFUN(AC_OCI8_VERSION,[
  AC_MSG_CHECKING([Oracle version])
  if test -s "$OCI8_DIR/orainst/unix.rgs"; then
	OCI8_VERSION=`grep '"ocommon"' $OCI8_DIR/orainst/unix.rgs | sed 's/[ ][ ]*/:/g' | cut -d: -f 6 | cut -c 2-4`
	test -z "$OCI8_VERSION" && OCI8_VERSION=7.3
  elif test -f $OCI8_DIR/lib/libclntsh.s?.9.0; then
        OCI8_VERSION=9.0
  elif test -f $OCI8_DIR/lib/libclntsh.s?.8.0; then
	OCI8_VERSION=8.1
  elif test -f $OCI8_DIR/lib/libclntsh.s?.1.0; then
	OCI8_VERSION=8.0
  elif test -f $OCI8_DIR/lib/libclntsh.a; then
    if test -f $OCI8_DIR/lib/libcore4.a; then
	OCI8_VERSION=8.0
    else
      OCI8_VERSION=8.1
    fi
  else
    AC_MSG_ERROR(Oracle-OCI8 needed libraries not found)
  fi
  AC_MSG_RESULT($OCI8_VERSION)
])

AC_DEFUN(AC_CHECK_OCI8,
[

  # Detect if we should enable OCI support
  # When ORACLE_HOME is set try, when --with-oci8 is set try, otherwise not!
  AC_ARG_WITH(oci8,[  --with-oci8=DIR         Compile with OCI8 support], [
    AC_MSG_CHECKING(for ORACLE home)
    if test x$withval = xyes; then
      withval="$ORACLE_HOME"
    fi
    if test -n "$withval"; then
      oci8_home="$withval"
    else
      oci8_home="$ORACLE_HOME"
    fi
    AC_MSG_RESULT($oci8_home)
  ]#,
#  [
#    if test -n "$ORACLE_HOME"; then
#    AC_MSG_CHECKING(for ORACLE home)
#    oci8_home="$ORACLE_HOME"
#    AC_MSG_RESULT($oci8_home)
#    fi
#  ]
)

  if test -n "$oci8_home"; then

    OCI8_DIR="$oci8_home"
    AC_OCI8_VERSION

    if test -d "$oci8_home/rdbms/public"; then
       INCLUDES="-I$oci8_home/rdbms/public $INCLUDES"
    fi
    if test -d "$oci8_home/rdbms/demo"; then
       INCLUDES="-I$oci8_home/rdbms/demo $INCLUDES"
    fi
    if test -d "$oci8_home/network/public"; then
       INCLUDES="-I$oci8_home/network/public $INCLUDES"
    fi
    if test -d "$oci8_home/plsql/public"; then
       INCLUDES="-I$oci8_home/plsql/public $INCLUDES"
    fi
    if test -d "$oci8_home/plsql/public"; then
       INCLUDES="-I$oci8_home/plsql/public $INCLUDES"
    fi
    if test -d "$oci8_home/include"; then
       INCLUDES="-I$oci8_home/include $INCLUDES"
    fi

    LCL_OCI8_LIBS="-L$oci8_home/lib $LCL_OCI8_LIBS"
    INCLUDES="-I$oci8_home/rdbms/demo $INCLUDES"
#    CXXFLAGS="-I$oci8_home/rdbms/demo $CXXFLAGS"

    AC_CHECK_HEADER(oci.h, [oci8_have_oci_h=yes] )

    AC_MSG_CHECKING(for ORACLE syslibs)
    if test -f "$oci8_home/lib/sysliblist"; then
      oci8_syslibs=`cat $oci8_home/lib/sysliblist`
    elif test -f "$oci8_home/rdbms/lib/sysliblist"; then
      oci8_syslibs=`cat $oci8_home/rdbms/lib/sysliblist`
    fi

    LCL_OCI8_LIBS="$oci8_syslibs $LCL_OCI8_LIBS"

    AC_MSG_RESULT($oci8_syslibs)

    case $OCI8_VERSION in
      9.0)
	LCL_OCI8_LIBS="-L$oci8_home/lib $LCL_OCI8_LIBS"
	;;
      8.1)
	LCL_OCI8_LIBS="-L$oci8_home/lib $LCL_OCI8_LIBS"
	;;
      *)
        AC_MSG_ERROR(Unsupported Oracle version!)
	;;
    esac

#    AC_CHECK_LIB(clntsh, OCIEnvInit, [oci8_have_libclntsh=yes])
    xlibs="$LIBS"
    LIBS="$LIBS $LCL_OCI8_LIBS"
    echo "$LIBS"
    AC_CHECK_LIB(clntsh, OCIEnvCreate, [LCL_OCI8_LIBS="$LCL_OCI8_LIBS -lclntsh"])
    LIBS="$xlibs"

    have_oci8=true

    AC_MSG_CHECKING(for OCI8)
      AC_DEFINE(HAVE_OCI,1,Define if we have OCI)
      AC_DEFINE(HAVE_OCI8,1,Define if we have OCI)
    AC_MSG_RESULT(yes)
  fi
  AM_CONDITIONAL(HAVE_OCI8, test x$have_oci8 = xtrue)
])

AC_DEFUN(AC_CHECK_NEED_STD,
[
  AC_MSG_CHECKING(checking if we need namespace std)
  AC_LANG_SAVE
  AC_LANG_CPLUSPLUS

  AC_TRY_COMPILE(
  [
  #include <iostream>
  ],
  [
  cout << "test" << endl;
  ],
  [AC_MSG_RESULT(no)],
  [
	AC_TRY_COMPILE(
	[
	#include <iostream>
	],
	[
	std::cout << "test" << std::endl;
	],
	[
	AC_MSG_RESULT(yes)
	AC_DEFINE(NEED_NAMESPACE_STD,1,Define if we need namespace std)
	],
	[
		AC_TRY_COMPILE(
		[
		#include <iostream.h>
		],
		[
		cout << "test" << endl;
		],
		[
		AC_MSG_RESULT(no)		
		],
		[AC_MSG_ERROR(Unable to detect std namespace)])
	])
  ])
  
  AC_LANG_RESTORE
])

AC_DEFUN(AC_DO_PCRE,
[
( cd pcre ; ./configure )
])

AC_DEFUN(AC_CHECK_PCRE,
[
  AC_CHECK_PROG(ac_have_pcre_config,pcre-config,yes)
  if test x$ac_have_pcre_config = xyes; then
    LIBLIBS="`pcre-config --libs` $LIBLIBS"
    CPPFLAGS="$INCLUDES `pcre-config --cflags`"
  else
    LIBLIBS="-lpcre $LIBLIBS"
  fi
  AC_CHECK_HEADER(pcre.h,[
    AC_DEFINE(HAVE_PCRE,1,Define if we have PCRE)
    ],
    [
    AC_MSG_RESULT(WARNING: You will need pcre to build LCL. Check into the subdirectory pcre and build it!)
    ]
    )
])

## Checks for sockets ....

AC_DEFUN(AC_CHECK_SOCK_CONNECT,
[
  AC_MSG_CHECKING(for constified connect)

  AC_LANG_SAVE
  AC_LANG_CPLUSPLUS

  AC_TRY_COMPILE(
[
  #include <sys/socket.h>
],
[
  struct sockaddr * addr;
  connect ( 0, (struct sockaddr const*)addr, 0 );
],
[

## this is const
AC_DEFINE(HAVE_CONST_CONNECT,1,Define if connect is constified)
AC_MSG_RESULT(yes)
],

[
AC_MSG_RESULT(no)
])


  AC_LANG_RESTORE
])

AC_DEFUN(AC_CHECK_TYPE_SOCK_DATATYPE,
[
  AC_MSG_CHECKING(for socket datatype)

  AC_LANG_SAVE
  AC_LANG_CPLUSPLUS

  AC_TRY_COMPILE(
[
#include <sys/types.h>
#include <sys/socket.h>
],
[
  #ifndef NULL
  #define NULL 0L
  #endif
  void * buffer;
  sendto ( 0, buffer, 0, 0, NULL, 0 );
],
[

## this is void
data_type="void *"
data_type_const="void const*"
AC_DEFINE(SOCK_DATATYPE_VOID,1,Define if the socket data type is void based)
],
[
## if not void, try char

  AC_TRY_COMPILE(
[
#include <sys/types.h>
#include <sys/socket.h>
],
[
  #ifndef NULL
  #define NULL 0L
  #endif
  char * buffer;
  sendto ( 0, buffer, 0, 0, NULL, 0 );
],
[

## this is char
data_type="char *"
data_type_const="char const*"
AC_DEFINE(SOCK_DATATYPE_CHAR,1,Define if the socket data type is char based)
],
[
AC_MSG_ERROR(unable to detect socket data type)
])

])

AC_LANG_RESTORE
AC_MSG_RESULT($data_type)
])

AC_DEFUN(AC_CHECK_BROKEN_FSTAT,
[
  AC_MSG_CHECKING(for broken fstat)
  AC_TRY_RUN([
  #include <sys/types.h>
  #include <sys/socket.h>
  #include <sys/stat.h>
  #include <unistd.h>
  #include <stdlib.h>

  void main (void)
  {
    int fd = socket ( AF_INET, SOCK_STREAM, 0 );
    struct stat si;
    if ( fstat ( fd, &si ) == -1 )
      exit ( 1 );
    if ( !S_ISSOCK ( si.st_mode ) )
      exit ( 2 );
    exit ( 0 );
  }
  ],
[
AC_MSG_RESULT(no)
],
[
AC_MSG_RESULT(yes)
AC_DEFINE(HAVE_BROKEN_FSTAT,1,Define if your fstat is not working corretly e.g. solaris 2.5)
],
[
AC_MSG_RESULT(no)
])
])

## END - Checks for sockets ....

AC_DEFUN(AC_CHECK_PF_MACROS,
[
  AC_MSG_CHECKING(for pretty function macros)
  AC_TRY_COMPILE(
    [
    ],
    [
    char const * tmp = __PRETTY_FUNCTION__
    ],
    [
    AC_MSG_RESULT(__PRETTY_FUNCTION__)
    AC_DEFINE(HAVE_PRETTY_FUNCTION_MACRO,1,Define if we have __PRETTY_FUNCTION__)
    ],
    [
    AC_MSG_RESULT(none)
    ]
    )
]
)

AC_DEFUN(AC_CHECK_TS_MACROS,
[
  AC_MSG_CHECKING(for timestamp macros)
  AC_TRY_COMPILE(
    [
    ],
    [
    char const * tmp =__TIMESTAMP__
    ],
    [
    AC_MSG_RESULT(__TIMESTAMP__)
    AC_DEFINE(HAVE_TIMESTAMP_MACRO,1,Define if we have __TIMESTAMP__)
    ],
    [
	AC_TRY_COMPILE(
	[
	],
	[
	char const * tmp = __TIME__ " " __DATE__;
	],
	[
	AC_MSG_RESULT(__TIME__ and __DATE__)
	AC_DEFINE(HAVE_TIMEANDDATE_MACRO,1,Define if we have __TIME__ and __DATE__)
	],
	[
	AC_MSG_RESULT(none)
	])
    ])

])

AC_DEFUN(AC_CHECK_XFUNC_USLEEP,
[
  AC_MSG_CHECKING(for usleep)
  AC_TRY_COMPILE(
	[
	#include <unistd.h>
	],
	[
	usleep ( 100 );
	],
	[
	AC_MSG_RESULT(yes)
	AC_DEFINE(HAVE_USLEEP,1,Define if we have usleep)
	],
	[
	AC_MSG_RESULT(no)
	])
])

AC_DEFUN(LCL_CHECK_PTHREAD_MUTEX,
[
  AC_MSG_CHECKING(for guard mutex definition)
  AC_TRY_COMPILE(
    [
    #include <pthread.h>
    ],
    [
    pthread_mutexattr_settype(0L,PTHREAD_MUTEX_RECURSIVE);
    ],
    [
    AC_MSG_RESULT(settype)
    AC_DEFINE(HAVE_PTHREAD_MUTEX_SETTYPE,1,Define if we have some unkown style NP standard)
    ],
    [
	save_CFLAGS="$CFLAGS"
	save_CXXFLAGS="$CFLAGS"
	CFLAGS="$CFLAGS -D_XOPEN_SOURCE=500"
	CXXFLAGS="$CXXFLAGS -D_XOPEN_SOURCE=500"
	AC_TRY_COMPILE(
	[
	#include <pthread.h>
	],
	[
	pthread_mutexattr_settype(0L,PTHREAD_MUTEX_RECURSIVE);
	],
	[
	AC_MSG_RESULT(settype with _XOPEN_SOURCE=500)
        AC_DEFINE(HAVE_PTHREAD_MUTEX_SETTYPE,1,Define if we have some unkown style NP standard)
	],
	[
	CFLAGS="$save_CFLAGS"
	CXXFLAGS="$save_CXXFLAGS"
	AC_MSG_RESULT(falling back to LCL recursive mutex emulation)
	AC_MSG_WARN(
You system has no operating system implementation of
a recursive mutex! LCL will provide the functionality
using an emulated version of this using normal mutex
locks! The behavior of the recursive mutex is the same
but using an operating sytem implementation is strongly
recommended!!)
	AC_DEFINE(NEED_FALLBACK_GUARD_MUTEX,1,Define if we need to use our own recursive mutex implementation)
	])

	])

    ])
])

AC_DEFUN(AC_CHECK_TYPE_SOCKLEN,
[
  AC_MSG_CHECKING(for socklen_t)
  AC_TRY_COMPILE(
[
#include <sys/socket.h>
],
[
  socklen_t sl;
],
[
AC_MSG_RESULT(yes)
AC_DEFINE(HAVE_SOCKLEN_T,1,[Define if we have socklen_t in sys/socket.h])
],
AC_MSG_RESULT(no)
)
])


AC_DEFUN(LCL_CHECK_LIBS,
[

  LCL_LIB_CPPFLAGS="-D_LCL_BUILD_LIB_"

  LCL_CHECK_PTHREAD_MUTEX

  AC_CHECK_PF_MACROS

  ac_win32="no"
  AC_ARG_ENABLE(win32,[  --enable-win32         Win32 Code], ac_win32="yes")

  ac_odbc32="yes"
  AC_ARG_ENABLE(odbc, [  --enable-odbc          ODBC32 Code], ac_odbc="$enableval")

  AC_ARG_WITH(mysql,[  --with-mysql         Compile with MySQL support], [
    mysql_libpath="$withval"
  ] )

  if test -n "$mysql_libpath"; then
    LIBS="-L$mysql_libpath $LIBS"
  fi
  AC_CHECK_LIB(mysqlclient, mysql_connect, [LIBMYSQLCLIENT="-lmysqlclient"; AC_DEFINE(HAVE_LIBMYSQLCLIENT,1,Define if we have mysqclient)] )

  AC_ARG_WITH(pluginpath, [  --with-plugin-path=path       Compile with PlugIn Path], [ LCL_PLUGIN_PATH="$withval" ], [LCL_PLUGIN_PATH='${prefix}/share/lcl' ] )

  AC_CHECK_LIB(pthread, pthread_create, [LIBPTHREAD="-lpthread"; AC_DEFINE(HAVE_LIBPTHREAD,1,Define if we have pthread)] )
  AC_CHECK_LIB(socket, socket)
  AC_CHECK_LIB(posix4, sem_init)
  AC_CHECK_LIB(nsl,gethostbyname)

  if test "$ac_odbc" != "no"; then
  AC_CHECK_LIB(odbc32, SQLConnect,
	LCL_ADD_LIBS="$LCL_ADD_LIBS -lodbc32"
	AC_DEFINE(HAVE_ODBC32,1,Define if we have ODBC32)
	ac_have_odbc32="yes"
      )
  fi
  AM_CONDITIONAL(HAVE_ODBC32, test x$ac_have_odbc32 = "xyes")


  AC_CHECK_HEADERS(sys/types.h sys/stat.h unistd.h)
  AC_CHECK_HEADERS(pthread.h)
  AC_CHECK_HEADERS(arpa/inet.h netdb.h netinet/in.h sys/socket.h)

  AM_CONDITIONAL(HAVE_WIN32, test x$ac_win32="xyes")

  AC_CHECK_TYPE_SOCKLEN
  AC_CHECK_TYPE_SOCK_DATATYPE
  AC_CHECK_SOCK_CONNECT
  AC_CHECK_BROKEN_FSTAT

  AC_CHECK_XFUNC_USLEEP

  AC_CHECK_OCI8

  AC_CHECK_LIB(jpeg, jpeg_start_compress)

  AC_CHECK_HEADER(dlfcn.h)
  AC_CHECK_LIB(dl,dlopen)

  AC_CHECK_HEADER(sys/utsname.h, AC_DEFINE(HAVE_UTSNAME_H,1,Define if we have sys/utsname.h) )
  AC_CHECK_FUNC(uname, AC_DEFINE(HAVE_UNAME,1,Define if we have uname) )
  AC_CHECK_FUNC(nanosleep, AC_DEFINE(HAVE_NANOSLEEP,1,Define if we have nanosleep) )
  AC_CHECK_FUNC(memmove, AC_DEFINE(HAVE_MEMMOVE,1,Define if we have memmove), AC_MSG_ERROR(Need some type of memmove up to now) )

  AC_CHECK_TS_MACROS

  AC_CHECK_PCRE

  AC_CHECK_SIZEOF(char)
  AC_CHECK_SIZEOF(short)
  AC_CHECK_SIZEOF(int)
  AC_CHECK_SIZEOF(long)
  AC_CHECK_TYPES([long long, long double, __int64])
  AC_CHECK_SIZEOF(long long)
  AC_CHECK_SIZEOF(__int64)
  AC_CHECK_SIZEOF(void *)
  AC_CHECK_SIZEOF(long double)

#  AC_CHECK_PROG(doxygen,ac_have_doxygen="yes")
  AC_CHECK_PROG(ac_have_doxygen,doxygen,"yes")


  AC_CHECK_NEED_STD

  AC_CHECK_HEADER(sstream,AC_DEFINE(HAVE_NEW_SSTREAM,1,Define if we have new sstream header))
  AC_CHECK_HEADER(strstream,AC_DEFINE(HAVE_NEW_STRSTREAM,1,Define if we have new strstream header))
  AC_CHECK_HEADER(iostream,AC_DEFINE(HAVE_NEW_IOSTREAM,1,Define if we have new iostream header))
  AC_CHECK_HEADER(fstream,AC_DEFINE(HAVE_NEW_FSTREAM,1,Define if we have new fstream header))

  # Finish it ...

  LCL_ADD_LIBS="$LIBPTHREAD $LIBMYSQLCLIENT $LCL_ADD_LIBS"
  LIBS="$LCL_ADD_LIBS $LIBS"
  all_libraries="$all_libraries $LIBPTHREAD $LIBMYSQLCLIENT"
  all_includes="$INCLUDES"

  AC_SUBST(all_libraries)
  AC_SUBST(all_includes)
  AC_SUBST(LCL_ADD_LIBS)
  AC_SUBST(LCL_OCI8_LIBS)
  AC_SUBST(LCL_PLUGIN_PATH)
  LCL_CFLAGS=$CFLAGS
  LCL_CXXFLAGS=$CXXFLAGS
  AC_SUBST(LCL_CFLAGS)
  AC_SUBST(LCL_CXXFLAGS)
  AC_SUBST(LCL_LIB_CPPFLAGS)
  AC_SUBST(INCLUDES)
  AC_SUBST(LIBLIBS)
])

AC_DEFUN(LCL_PROG_LIBTOOL,
[
AC_REQUIRE([AC_ENABLE_SHARED])
AC_REQUIRE([AC_ENABLE_STATIC])
rtk_save_cc=$CC
rtk_save_cflags="$CFLAGS"
CC=$CXX
CFLAGS="$CXXFLAGS"
AC_LANG_SAVE
AC_LANG_CPLUSPLUS
AC_PROG_LIBTOOL
LIBTOOL="$LIBTOOL --silent"
AC_LANG_RESTORE
CC=$rtk_save_cc
CFLAGS="$rtk_save_cflags"
])
