/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

/* LCL - The Linux Class Library
 * Copyright (C) 1998-2003 Jens Reimann
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _LCL_H_
#define _LCL_H_

#ifndef _REENTRANT
#define _REENTRANT
#define _THREAD_SAFE
#endif

typedef int LFileDescriptor;
#define LINVALID_FILEDESC -1

// do some detection of where and what we are

#define _LCL_BUILD_ENV_ 1

#ifdef _MSC_VER
# define _LCL_COMPILER_MSVC_
#endif

#ifdef __DECCXX
# define _LCL_COMPILER_DECCXX_
#endif

#ifdef __NUTC__
# define _LCL_ENV_NUTCRACKER_
# define _LCL_SYS_POSIX_
#endif

#ifdef SAG_COM
# define _LCL_ENV_SAGDCOM_
#endif


#ifdef _WIN32
# define _LCL_SYS_WIN32_
# ifdef _LCL_ENV_SAGDCOM_
#  undef _LCL_SYS_WIN32_
#  define _LCL_SYS_POSIX_
# endif
#else
# define _LCL_SYS_POSIX_
#endif



#if defined ( _LCL_SYS_WIN32_ ) && ( ! defined ( _LCL_ENV_NUTCRACKER_ ) ) && ( ! defined ( _LCL_ENV_SAGDCOM_ ) )
# define _LCL_SYS_WIN32_PURE_
#endif

#ifdef _LCL_COMPILER_MSVC_
#define _LCL_EXPORT_TAG_ __declspec(dllexport)
#define _LCL_IMPORT_TAG_ __declspec(dllimport)
#endif

// Include Everything which is needed for MS C++

#ifdef _LCL_COMPILER_MSVC_
# pragma warning (disable: 4250)
# pragma warning (disable: 4786)
# pragma warning (disable: 4251)
# pragma warning (disable: 4355)

# define access _access

#endif

// POSIX
#ifdef _LCL_SYS_POSIX_
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#endif

// Pure WIN32
#ifdef _LCL_SYS_WIN32_PURE_

#define _LCL_OPT_FOPEN_BIN_
/*
 * We are now defaulting to WINNT/2k/XP ... therefore we already have winsock2
 */
//#include <winsock2.h>

  #include <io.h>
  #define R_OK 0x04

// Using MFC?
  #ifdef __AFX_H__
    #include <afxsock.h>
  #else
    #include <windows.h>
  #endif

  typedef DWORD LThreadID;
  typedef HMODULE LLibraryHandle;
  #define LLIBRARY_INVALID NULL

  #define PURE_WIN32

#else

  #include <dlfcn.h>

  typedef void* LLibraryHandle;
  #define LLIBRARY_INVALID NULL

#endif

// Assume Posix
#ifdef _LCL_SYS_POSIX_

// Basic
#  include <fcntl.h>
#  include <sys/stat.h>
#  include <sys/types.h>
#  include <unistd.h>

// Sockets
#  include <arpa/inet.h>
#  include <netdb.h>
#  include <netinet/in.h>
#  include <sys/socket.h>

// Threads
#  include <pthread.h>

// Typedefs

typedef pthread_t LThreadID;

#endif

#ifdef _LCL_SYS_WIN32_
#  include <windows.h>
#endif

// Windows DLL Export

#ifdef _LCL_SYS_WIN32_

  #ifdef BUILD_STATIC
    #define LCL_EXPORT
  #else
    #ifdef BUILD_LCL_DLL
      #define LCL_EXPORT __declspec(dllexport)
    #else
      #define LCL_EXPORT __declspec(dllimport)
    #endif
  #endif

  #ifdef BUILD_LCL_DLL_PLUGIN
    #define LCL_EXPORT_PLUGIN __declspec(dllexport)
  #else
    #define LCL_EXPORT_PLUGIN __declspec(dllimport)
  #endif

#else
  #define LCL_EXPORT
  #define LCL_EXPORT_PLUGIN
#endif

// Old macros

#ifndef FALSE
   #define FALSE 0
   #define TRUE 1
#endif

#ifndef ISORNOT
   #define ISORNOT(s) (s ? "" : "not")
#endif

#ifndef YESNO
   #define YESNO(s) (s ? "yes" : "no")
#endif

#ifndef CLASSNAME
# define CLASSNAME(x)
#endif

#ifndef MIN
   #define MIN(v1,v2) ((v1>v2) ? v2 : v1)
#endif

#ifndef MAX
   #define MAX(v1,v2) ((v1>v2) ? v1 : v2)
#endif

// Workaround for "functions needs to return..." after throw bug of MSVC

#ifndef MSTHROW
   #ifdef _LCL_COMPILER_MSVC_
      #define MSTHROW(v) return *(( v *)NULL)
   #else
      #define MSTHROW(v)
   #endif // _WIN32
#endif // MSTHROW

#ifndef INADDR_NONE
 #define INADDR_NONE             ((uint32_t) 0xffffffff)
#endif

#include <lcl/lcl_config.h>

#ifndef HAVE_MODE_T
#define mode_t int
#endif

#ifdef HAVE_SOCKLEN_T
#define L_OPTLEN_T socklen_t
#define L_ADDRLEN_T socklen_t
#else
#define L_OPTLEN_T int
#define L_ADDRLEN_T int
#endif

#ifdef SOCK_DATATYPE_VOID
#define L_SOCK_DATA_TYPE void *
#define L_SOCK_DATA_TYPE_CONST void const *
#endif

#ifdef SOCK_DATATYPE_CHAR
#define L_SOCK_DATA_TYPE char *
#define L_SOCK_DATA_TYPE_CONST char const *
#endif

#ifdef ENABLE_DEBUG
#define LEMIT_INLINE
#else
#define LEMIT_INLINE inline
#endif

#include <lcl/lcl_types.h>

#include <sys/types.h>

#include <time.h>

#ifdef __cplusplus

#ifdef HAVE_NEW_SSTREAM
#  include <sstream>
#else
#  ifdef HAVE_NEW_STRSTREAM
#    include <strstream>
#  else
#    include <strstream.h>
#  endif
#endif

#ifdef HAVE_NEW_IOSTREAM
#include <iostream>
#else
#include <iostream.h>
#endif

#ifdef HAVE_NEW_FSTREAM
#include <fstream>
#else
#include <fstream.h>
#endif

#ifdef NEED_NAMESPACE_STD
#define NAMESPACE_STD(c) std::c
#else
#define NAMESPACE_STD(c) c
#endif

#ifdef HAVE_TIMESTAMP_MACRO
#  define __LCL_TIMESTAMP__ __TIMESTAMP__
#endif

#ifdef HAVE_TIMEANDDATE_MACRO
#  define __LCL_TIMESTAMP__ (__TIME__ " " __DATE__)
#endif

#ifndef __LCL_TIMESTAMP__
#  define __LCL_TIMESTAMP__ "<unsupported>"
#endif

#endif /* __cplusplus */

#endif /* _LCL_H_ */

