/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

/* ****************************************************************************
   |      File: lapp.h
   |      Contains: class definition for LApplication
   |      Created: 15.02.1998
   |      Last Changed: 04.03.1998
   |      Version: 0.0
   |         
   |      ToDo: + just started
   |            + add some exceptions
   |
   ************************************************************************* */

/* ====>> See also the ChangeLog in lapp.cc <<==== */

#ifndef _L_APP_H_
#define _L_APP_H_

#include <lcl.h>

#define LDEBUG(msg) if ( theApp.debugFlag ) theApp.printDebug ( (msg) )

#include <lcl/lobject.h>
#include <lcl/lstring.h>

#include <lcl/lmap.h>
#include <lcl/lheap.h>
#include <lcl/lset.h>
#include <lcl/lhash.h>

/*!
 * \class LApplication
 * \author Jens Reimann
 * \brief Application control class
 *
 * LApplication is a generic application control class. There may only be one instance of
 * this class at a time which is automatically created on application startup. It is accessable
 * using the global variable \em lclApp
 *
 * LApplication must be initialized before it may be used. To do this simply call
 * LApplication::init which will initialize the already created instance. You should to this
 * as your first step before doing anything else. This will initialize the trace and log system and
 * the application arguments (which may be queried using isParam, hasParam)
 *
 * If you want LApplcation to process the internal --lcl command line arguments you must also
 * call processLCLParams after init 
 *
 */

class LCL_EXPORT LApplication : public LObject {

protected:

  LString progName;
  LHash<LString,LString> paramDict;
  LHeap<LString> paramFiles;
  LString argStr;

  LString myBuildVersion, myBuildHost, myBuildDate;
  long myBuildNr;

  void setTracing ( LString const & param );

public:

  static NAMESPACE_STD(ostream) & logStream;
  static bool debugFlag;

  bool callExit;
  bool isDestructing;

  LApplication ();

  /*!
   * \name Basic debug output
   *
   * This system should only be used for internal debugging. For an appliction style
   * tracing please refer to the module "Tracing"!
   */
  //@{
  void printDebug ( LString const & );
  //@}

  /*!
   * \name Initialzation
   */
  //@{
  void init ( int argc, char * argv[] ); //!< Initializes the already created LApplcation instance
  void initMyVersion ( LString const & version, LString const & date = "", LString const & host = "", long build_nr = 0 );
  void processLCLParameters (); //!< processes LCL specific command line parameters
  void showLCLParametersSyntax ();
  //@}


  // ************ Parameter funcs ****************
  /*!
   * \name Parameter Functions (Command Line Arguments)
   */
  //@{
  //! Return the value of a parameter
  LString getParam ( LString key, LString def = LString() );
  //! Is a parameter set
  bool isParam ( LString const & key );
  //! Is a parameter set with value.
  bool hasParam ( LString const & key );
  LHeap<LString> getFileList();
  LString args(); //!< returns all command line arguments in one string
  LString appName () const; //!< returns the stripped application name 
  //@}

  // ************ Version funcs ******************

  virtual ~LApplication ();
};

#ifndef _L_APP_CC_
LCL_EXPORT extern LApplication theApp;
#endif

#endif
