/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

#ifndef _LCOLOR_H_
#define _LCOLOR_H_

#include <lcl.h>
#include <lcl/lstring.h>

/*!
 * \class LColor
 * \brief A simple color class
 * \author Jens Reimann <ctron@dentrassi.de>
 *
 * Implements a color class which manages a 4 channel (red, green, blue, alpha) color code.
 * All channels have 32 bit (which seems to be a bit much, but that's ok *gg*)
 */
class LCL_EXPORT LColor {
 protected:
  unsigned long red, green, blue, alpha;
  bool valid;

 public:
  LColor ();
  LColor ( unsigned char _red, unsigned char _green, unsigned char _blue, unsigned char _alpha = 0 );
  LColor ( int _red, int _green, int _blue, int _alpha = 0 );
  LColor ( unsigned long _red, unsigned long _green, unsigned long _blue, unsigned long _alpha = 0L );
  LColor ( double _red, double _green, double _blue, double _alpha = 0L );

  unsigned long getRed () const;
  unsigned long getGreen () const;
  unsigned long getBlue () const;
  unsigned long getAlpha () const;

  unsigned char getRedAsChar () const;      //!< Get the 8 bit red component of the color
  unsigned char getGreenAsChar () const;    //!< Get the 8 bit green component of the color
  unsigned char getBlueAsChar () const;     //!< Get the 8 bit blue component of the color
  unsigned char getAlphaAsChar () const;    //!< Get the 8 bit alpha component of the color

  unsigned short getRedAsWord () const;
  unsigned short getGreenAsWord () const;
  unsigned short getBlueAsWord () const;
  unsigned short getAlphaAsWord () const;

  void setRed ( unsigned char _red );       //!< Set the 8 bit red component of the color
  void setGreen ( unsigned char _green );   //!< Set the 8 bit green component of the color
  void setBlue ( unsigned char _blue );     //!< Set the 8 bit blue component of the color
  void setAlpha ( unsigned char _alpha );   //!< Set the 8 bit alpha component of the color

  LString asHexRGB8 () const;
  LString asHexRGBA8 () const;

  LString asHexRGB16 () const;
  LString asHexRGBA16 () const;

  bool operator == ( LColor const & ) const;
  bool operator != ( LColor const & ) const;

  bool isValid () const;
  void setValid ( bool = true );

  LColor asGray () const;
  void toGray ();
};

#endif
