/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

/* ***************************************************************************
   |      File: lconfsys.h
   |      Contains: case class LConfigSystem
   |      Created: 19.03.1998
   |      Last Changed: 19.03.1998
   |      Version: 0.0
   |         
   |      ToDo: + just started
   |
   ************************************************************************* */

/* ====>> See also the ChangeLog in lconfsys.cc <<==== */

#ifndef _L_CONFSYSTEM_H_
#define _L_CONFSYSTEM_H_

#include <lcl.h>

class LCL_EXPORT LConfigSystem;

#include <lcl/lobject.h>
#include <lcl/lconfparser.h>
#include <lcl/lheap.h>
#include <lcl/lconfobj.h>
#include <stdio.h>
#include <lcl/literator.h>
#include <lcl/lpointer.h>

#include <lcl/lconfmodule.h>

/**
 * \class LConfigSystem
 * \brief Configuration data system
 * \author Jens Reimann <ctron@dentrassi.de>
 * \version 0.2
 * \date 2000
 * \bug Should be more or less bug free. Need to implement a tree structure some time.
 *
 *
 * This is a generic class which holds configuration data of different types. Data is structured
 * in objects, each object can contain different attributes. Each attribute has a value assigned.
 * The structure is filled by \e parsers like LDefaultConfigParser of LCGIParser.
 *
 * It is also possible to derive your own parser (data fetcher) from LConfigParser
 */

class LCL_EXPORT LConfigSystem : public LObject {

 protected:
  LHeap <LConfigObject> objList;

 public:  
  LConfigSystem ( LConfigParser * );        //!< Creates a new instance (depricated)
  LConfigSystem ( LConfigReader const & );  //!< Creates a new instance and reads storage
  LConfigSystem ( );
  ~LConfigSystem ();

  void read ( LConfigParser * );

  void clear ();

  void addObject ( LConfigObject & obj );   //!< Add a new object to the system
  unsigned long count ( LString const & );  //!< Counts the objects of a given type

  //! Iterate through all objects
  void forEach ( LIterator<LConfigObject> & ); 
  //! Iterate through all objects
  void forEach ( void (*func)(LConfigObject&, void*), void * ptr );
  //! Iterate through all objects of a specific type
  void forEachType ( LString const &, LIterator<LConfigObject> & );
  //! Iterate through all objects of a specific type
  void forEachType ( LString const &, void (*func)(LConfigObject&, void*), void * ptr );

  //! Get the first object of the given type
  LConfigObject & first ( LString const & type );
  //! Get the first object of the given type where a attribute value matches
  LConfigObject & firstObjectWithMemberEqual ( LString const & _type, LString const & _member, LString const & _value, bool casesensitive = true );

  //! Check for an object of the given type where a attribute value matches
  bool containsObjectWithMemberEqual ( LString const & _type, LString const & _member, LString const & _value, bool casesensitive = true );
};

#endif
