#ifndef _LFD_LAYER_WIN32_H_
#define _LFD_LAYER_WIN32_H_

#include <lcl.h>
#include <lcl/lvcs.h>
#include <lcl/lxcpt.h>
#include <lcl/lpointer.h>
#include <lcl/lmutex.h>
#include <lcl/lhash.h>

#include <lcl/lwin32.fd_basic.h>

#include <lcl/lwin32.select.h>

class LCL_EXPORT LFDManager {
  LVCS_DECLARE;

  friend class LWin32Select;

private:
  LGuardMutex fdHashLock;
  LHash < LFileDescriptor, LManagedPointer < LFDManager_Basic > > fdHash;

protected:

  static LMutex instanceLock;
  static LFDManager * theInstance;   // Common
  LFDManager ();

  LFileDescriptor registerHandle ( LManagedPointer<LFDManager_Basic> );
  void unregisterHandle ( LFileDescriptor );

  LManagedPointer<LFDManager_Basic> getHandler ( LFileDescriptor ) const;

 public:

  enum OpenFlags {  read = 0x01, write = 0x02, append = 0x4, create = 0x08, failExists = 0x10, truncate = 0x20 };

  static LFDManager * getInstance ();  // Common

  bool is_valid ( LFileDescriptor fd ) const;
  bool is_socket ( LFileDescriptor fd ) const;

  int sys_setsockopt ( LFileDescriptor fd, int level, int optname, void const * optval, unsigned long optlen );
  int sys_getsockopt ( LFileDescriptor fd, int level, int optname, void * optval, unsigned long *optlen );

  int sys_open ( char const * name, int of, mode_t om );

  int sys_socket ( int af, int type, int proto );
  int sys_bind ( LFileDescriptor fd, struct sockaddr const * addr, unsigned long addr_len );
  int sys_connect ( LFileDescriptor fd, struct sockaddr const * addr, unsigned long addr_len );
  int sys_listen ( LFileDescriptor fd, int backlog );
  LFileDescriptor sys_accept ( LFileDescriptor fd, struct sockaddr * addr, unsigned long * addr_len );
  int sys_getsockname ( LFileDescriptor fd, struct sockaddr *name, unsigned long *namelen );
  int sys_getpeername ( LFileDescriptor fd, struct sockaddr *name, unsigned long *namelen );

  int sys_sendto ( LFileDescriptor fd, void const * data, unsigned long len, int flags, struct sockaddr const *to, unsigned long tolen );
  int sys_recvfrom ( LFileDescriptor fd, void * data, unsigned long len, int flags, struct sockaddr *from, unsigned long * fromlen );

  LFileDescriptor sys_dup ( LFileDescriptor fd );
  int sys_read ( LFileDescriptor fd, void * ptr, unsigned long size );
  int sys_write ( LFileDescriptor fd, void const * ptr, unsigned long size );
  int sys_close ( LFileDescriptor fd );

  bool sys_getblocking ( LFileDescriptor fd );
  void sys_setblocking ( LFileDescriptor fd, bool state );

  bool sys_select ( LSet<LFileDescriptor> const & read,
		    LSet<LFileDescriptor> const & write,
		    LSet<LFileDescriptor> const & xcpt,
		    LSet<LFileDescriptor> & read_res,
		    LSet<LFileDescriptor> & write_res,
		    LSet<LFileDescriptor> & xcpt_res,
		    unsigned long msecs );
};

#endif
