#ifndef _L_FDSENDQUEUE_H_
#define _L_FDSENDQUEUE_H_

#include <lcl/lfiledesc.h>
#include <lcl/liostream.h>
#include <lcl/lref.h>
#include <lcl/lheap.h>

class LFDSendQueue;
class LFDSendQueueStorage;

class LFDSendQueueStream : public LOStream {
  friend class LFDSendQueue;

 protected:

  LManagedPointer<LFDSendQueueStorage> storage;

 public:

  LFDSendQueueStream ( LManagedPointer<LFDSendQueueStorage> );

  ~LFDSendQueueStream ();

  void processSend ( LFileDesc * );

  virtual bool mayWrite ();
  virtual int write ( char c );
  virtual int write ( void const *, unsigned long len );

  void close ();
};

class LFDSendQueueStorage {
 protected:
  friend class LFDSendQueue;

  LFDSendQueue & myQueue;
  LReference<LFileDesc> fd;

  LFDSendQueueStorage ( LFDSendQueue &, LReference<LFileDesc> );

  LBuffer buffer;

  bool valid, closed;

 public:

  ~LFDSendQueueStorage ();

  bool isValid () const;
  bool isClosed () const;

  void append ( void const * ptr, unsigned long len );  
  void processSend ( LFileDesc * );

  void close ( );

};

class LFDSendQueue {
  LVCS_DECLARE;

 protected:
  LHash < LReference < LFileDesc >, LManagedPointer<LFDSendQueueStorage> > fdHash;

  void cleanUp ();

  class CleanUpIter : public LHashIterator < LReference < LFileDesc >, LManagedPointer<LFDSendQueueStorage> > {    
  public:
    LHeap < LReference < LFileDesc > > cleanList;
    void applyTo ( LReference < LFileDesc > & fd, LManagedPointer<LFDSendQueueStorage> & fs);
  };

 public:
  LManagedPointer<LFDSendQueueStorage> getStorage ( LReference<LFileDesc> );

  LSelectLoop * getLoop () const;

  void afterWriteEvent ( LFileDesc * fd );
};

#endif
