#ifndef _LHTTP_H_
#define _LHTTP_H_

#include <lcl/lvcs.h>
#include <lcl/lstring.h>
#include <lcl/lhash.h>
#include <lcl/liostream.h>

class LCL_EXPORT LHTTPBase {
  LVCS_DECLARE;
 protected:
  LHash<LString,LString> vars;

 public:
  LHash<LString,LString> & getVars ();

  void setContentType ( LString const & = LString("text/html") );
  void setContentType ( LString const & p1, LString const & p2 );
  LString getContentType ( ) const;
};

class LCL_EXPORT LHTTPRequest : public LHTTPBase {
  LVCS_DECLARE;

 protected:
  LString version, uri, method;

 public:

  LHTTPRequest ( LString const & uri, LString const & method = "GET",  LString const & version = "HTTP/1.1" );
  LHTTPRequest ( LIStream & );

  bool readFromStream ( LIStream & );

  LString getMethod () const;
  LString getURI () const;
  LString getVersion () const;
  void setMethod ( LString const & );
  void setVersion ( LString const & );
  void setURI ( LString const & );

};


class LCL_EXPORT LHTTPResponse : public LHTTPBase {
  LVCS_DECLARE;

 protected:
  long status;
  LString version, reason;

 public:

  LHTTPResponse ( long status, LString const & reason = LString(), LString const & version = "HTTP/1.1" );
  LHTTPResponse ( LIStream & );

  bool readFromStream ( LIStream & );
  bool sendToStream ( LOStream & );

  long getStatus () const;
  LString getReason () const;
  LString getVersion () const;
  void setStatus ( long );
  void setReason ( LString const & );
  void setVersion ( LString const & );

};

#endif
