#ifndef _LHTTP_DIRECTOR_H_
#define _LHTTP_DIRECTOR_H_

#include <lcl.h>

class LCL_EXPORT LHTTPDirector;
class LCL_EXPORT LHTTPProcessor;

#include <lcl/lstring.h>

LCL_EXPORT LString key ( LHTTPProcessor * );

#include <lcl/lvcs.h>
#include <lcl/lhttp.h>
#include <lcl/lheap.h>
#include <lcl/lmap.h>
#include <lcl/lsocket.h>

/*!
 * \class LHTTPProcessor
 * \brief Virtual base class of a HTTP processor
 */

class LCL_EXPORT LHTTPProcessor {
  friend LString LCL_EXPORT key ( LHTTPProcessor * );
  friend class LCL_EXPORT LHTTPDirector;
  LVCS_DECLARE;

 public:
  class LCL_EXPORT Info {
  protected:
    LString name;
    LString description;
    LString author;
    LString version;
    LString entryPoint;
  public:
    void setName ( LString const & );
    void setDescription ( LString const & );
    void setAuthor ( LString const & );
    void setVersion ( LString const & );
    void setEntryPoint ( LString const & );

    LString getName () const;
    LString getDescription () const;
    LString getAuthor () const;
    LString getVersion () const;
    LString getEntryPoint () const;
  };

 protected:
  LString expression;
  bool async;

  void setExpression ( LString const & );
  bool match ( LString const & uri ) const;
  void preProcess ( LHTTPRequest, LSocket4Inet & );

 public:

  LHTTPProcessor ();

  void setAsync ( bool flag = true );  
  virtual void process ( LHTTPRequest, LSocket4Inet & ) = 0;

  virtual Info getInformation () const;

  LString getExpression () const;
};

/*!
 * \class LHTTPXProcessor
 * \brief A exception safe version of LHTTPProcessor
 */

class LCL_EXPORT LHTTPXProcessor : public LHTTPProcessor {
 protected:
 public:
  virtual void process ( LHTTPRequest, LSocket4Inet & );
  
  virtual void saveProcess ( LHTTPRequest const & req, LSocket4Inet & ) = 0;
};


/*!
 * \class LHTTPDirector
 * \author Jens Reimann
 * \brief HTTP redirection to request processors based on URL
 */

class LCL_EXPORT LHTTPDirector {
  LVCS_DECLARE;

 protected:
  LMap<LHTTPProcessor *, LString> procList;
  LHTTPProcessor * errorProc;

 public:
  LHTTPDirector ();

  LHeap<LHTTPProcessor::Info> getInfoList ( bool skipDummies = true ) const;

  void processRequest ( LHTTPRequest, LSocket4Inet * );

  void addProcessor ( LString const &, LHTTPProcessor & );
  void removeProcessor ( LString const & );

  void setErrorProcessor ( LHTTPProcessor & );
  
};

#endif
