#ifndef _LIMAGE_H_
#define _LIMAGE_H_

#include <lcl/lcolor.h>
#include <lcl/ltable.h>
#include <lcl/liostream.h>
#include <lcl/lbuffer.h>

#include <lcl/lstreamproxy.h>

class LCL_EXPORT LColorMap : public LTable<LColor> {
 protected:
 public:
  LColorMap ();
  LColorMap ( LColorMap const & );
  LColorMap ( unsigned long );

  unsigned long findColor ( LColor const & );
  bool containsColor ( LColor const & );
};

class LImage;

class LCL_EXPORT LImageReader {
 protected:
 public:
  bool read ( LImage & image, LIStream & stream ) const
    {
      try {
	return impl_read ( image, stream );
      }
      catch ( ... ) {
	return false;
      }
    }
  virtual bool impl_read ( LImage &, LIStream & ) const = 0;
};

class LCL_EXPORT LImageWriter {
 protected:
 public:
  
  bool write ( LImage const & image, LOStream & stream ) const
    {
      try {
	return impl_write ( image, stream );
      }
      catch ( ... ) {
	return false;
      }
    }
  virtual bool impl_write ( LImage const &, LOStream & ) const = 0;
};

class LCL_EXPORT LImage {
 protected:
  unsigned long height, width;
  unsigned long bpp;
  LBuffer bits;
  LColorMap colorMap;

  bool valid;
  
 public:

  LImage ();
  LImage ( unsigned long _width, unsigned long _height, unsigned long _bpp );
  LImage ( unsigned long _width, unsigned long _height, unsigned long _bpp, LColorMap const & _map );

  void setColorMap ( LColorMap const & map );
  void setBitsPerPixel ( unsigned long );
  void setSize ( unsigned long _width, unsigned long _height );        //!< Resize the image. Will clear the content!!

  unsigned long getHeight () const;
  unsigned long getWidth () const;
  unsigned long getBitsPerPixel () const;
  LColorMap & getColorMap ();
  LColorMap const & getColorMap () const;

  void setBits ( LBuffer const & );

  LColor getPixel ( unsigned long x, unsigned long y ) const;
  void putPixel ( unsigned long x, unsigned long y, LColor const & color );

  bool isValid () const;                //!< Contains a valid image?

  bool read ( LImageReader const &, LIStream & );
  bool write ( LImageWriter const &, LOStream & ) const;
};

#endif
