#ifndef _LCL_QUEUE_H_
#define _LCL_QUEUE_H_

#include <lcl.h>

#include <lcl/lfifo.h>
#include <lcl/lmutex.h>
#include <lcl/lsem.h>
#include <lcl/lpointer.h>
#include <lcl/lset.h>

template < class Type >
class LCL_EXPORT LQueueMux;
template < class Type >
class LCL_EXPORT LQueueSender;
template < class Type >
class LCL_EXPORT LQueueReceiver;

class LCL_EXPORT LQueueFullException : public LBasicException {
protected:
public:
  LQueueFullException ( const char * _filename, unsigned long _line );
  virtual void log () const;
};

template < class Type >
class LCL_EXPORT LQueueSender {
protected:
  LGuardMutex controlLock;

  LQueueReceiver<Type> * m_receiver;
  virtual void inject ( Type const & );
 public:
  LQueueSender ( );
  LQueueSender ( LQueueReceiver<Type> & );
  void setReceiver ( LQueueReceiver<Type> & );
  LQueueReceiver<Type> & getReceiver () const;

  virtual void send ( Type const & );
};

template < class Type >
class LCL_EXPORT LQueueReceiver {
  friend class LQueueSender < Type >;
  friend class LQueueMux < Type >;
 protected:
  virtual void injector ( Type const & ) = 0;
 public:
};

template < class Type >
class LCL_EXPORT LQueueMux : public LQueueReceiver<Type> {
 protected:
  LGuardMutex muxLock;
  LSet < LQueueReceiver<Type> * > m_receiverList;
  virtual void injectTo ( LQueueReceiver<Type> &, Type const & );
  virtual void mux ( Type const & ) = 0;
  virtual void injector ( Type const & );
 public:
  void addReceiver ( LQueueReceiver<Type> & );
  void removeReceiver ( LQueueReceiver<Type> & );
};

// LQueueSender<Type>

template < class Type >
LQueueSender<Type>::LQueueSender ( ) :
  m_receiver ( NULL )
{
}

template < class Type >
LQueueSender<Type>::LQueueSender ( LQueueReceiver<Type> & receiver ) :
  m_receiver ( &receiver )
{
}

template < class Type >
void LQueueSender<Type>::setReceiver ( LQueueReceiver<Type> & receiver )
{
  LAutoLock _lock ( controlLock );
  m_receiver = &receiver;
}

template < class Type >
LQueueReceiver<Type> & LQueueSender<Type>::getReceiver ( ) const
{
  LAutoLock _lock ( controlLock );
  return *m_receiver;
}

template < class Type >
void LQueueSender<Type>::send ( Type const & item )
{
  LAutoLock _lock ( controlLock );
  inject ( item );
}

template < class Type >
void LQueueSender<Type>::inject ( Type const & item )
{
  if ( m_receiver != NULL )
    {
      m_receiver->injector ( item );
    }
}

// LQueueMux<Type>
template < class Type >
void LQueueMux<Type>::injector ( Type const & item )
{
  LAutoLock _lock ( muxLock );
  mux ( item );
}

template < class Type >
void LQueueMux<Type>::addReceiver ( LQueueReceiver<Type> & receiver )
{
  LAutoLock _lock ( muxLock );
  m_receiverList.add ( &receiver );
}

template < class Type >
void LQueueMux<Type>::removeReceiver ( LQueueReceiver<Type> & receiver )
{
  LAutoLock _lock ( muxLock );
  m_receiverList.removeItem ( &receiver );
}

template < class Type >
void LQueueMux<Type>::injectTo ( LQueueReceiver<Type> & receiver, Type const & item )
{
  receiver.injector ( item );
}

#endif
