
/* ****************************************************************************
   |      File: lsocket.h
   |      Contains: class definition for LSocket
   |      Created: 05.03.1998
   |      Last Changed: 25.02.2000
   |      Version: 0.0
   |         
   |      ToDo: + rewriting
   |
   ************************************************************************* */

/* ====>> See also the ChangeLog in lsocket.cc <<==== */

#ifndef _L_SOCKET_H_
#define _L_SOCKET_H_

class LSocket;
class LSocket4Inet;

#include <lcl.h>

#include <lcl/lfiledesc.h>
#include <lcl/lsockaddr.h>
#include <lcl/lsockaddr.4inet.h>

#ifndef _WIN32
#include <lcl/lsockaddr.unix.h>
#endif

/*!
 * \ingroup FileDescriptors
 * \author Jens Reimann <ctron@dentrassi.de>
 * \class LSocket
 * \brief A base class for all types of sockets
 */

class LCL_EXPORT LSocket : public LFileDesc {
 protected:
  int domain, type, protocol;

  //  static bool isSocket ( LFileDescriptor _fd );

  //  static LFileDescriptor socket ( int _domain, int _type, int _proto );

 public:
 
  LSocket ( int _domain, int _type, int _proto = 0 );
  LSocket ( LFileDescriptor _fd );

  void listen ( long = 10 );
  
  bool connect ( LSockAddr const & );
  void bind ( LSockAddr const & );
  virtual LSocket * accept () = 0;

  void reuseaddr ( bool state = true );

  int setsockopt ( int, int, const void * , int );
  void linger ( long );

  int sendto ( void const *, int len, sockaddr const * addr, unsigned long addrlen, int flags = 0  );
  int sendto ( LBuffer const &, LSockAddr const &, int flags = 0 );
  int recvfrom ( void * ptr, int len, sockaddr * addr, unsigned long * addrlen, int flags = 0 );  
};

/*!
 * \ingroup FileDescriptors
 * \author Jens Reimann <ctron@dentrassi.de>
 * \class LSocket4Inet
 * \brief A class for IPv4 (AF_INET) sockets
 */

class LCL_EXPORT LSocket4Inet : public LSocket {
 protected:
  static bool isConnected ( LFileDescriptor _fd );

 public:

  LSocket4Inet ( int _domain, int _type, int _proto = 0 );
  LSocket4Inet ( LFileDescriptor _fd );

  LSockAddr4Inet localAddr ();
  LSockAddr4Inet remoteAddr ();

  virtual LSocket * accept ();

  bool isConnected () const;

  long getType ();

  int recvfrom ( LBuffer &, LSockAddr4Inet &, int flags = 0 );

  bool addMembership ( LSockAddr4Inet const & group, LSockAddr4Inet const & addr = LSockAddr4Inet::any() );
};

/*!
 * \ingroup FileDescriptors
 * \author Jens Reimann <ctron@dentrassi.de>
 * \class LSocketUnix
 * \brief A class for Unix (AF_UNIX) sockets
 */

#ifndef _WIN32
class LSocketUnix : public LSocket {
 protected:
  static bool isConnected ( LFileDescriptor _fd );

 public:

  LSocketUnix ( int _domain, int _type, int _proto = 0 );
  LSocketUnix ( LFileDescriptor _fd );

  LSockAddrUnix localAddr ();
  LSockAddrUnix remoteAddr ();

  virtual LSocket * accept ();

  bool isConnected () const;

  long getType ();

  int recvfrom ( LBuffer &, LSockAddrUnix &, int flags = 0 );

};
#endif

#endif
