#ifndef _LSOCKETSERVER_H_
#define _LSOCKETSERVER_H_


#include <lcl.h>
#include <lcl/lsocket.h>
#include <lcl/lemit.h>
#include <lcl/lvcs.h>

/*!
 * \class LSocketServer
 * \author Jens Reimann
 * \brief A socket server class
 * \bugs Yes, there are some ;-) ... limited to ipv4, TCP and UDP
 *
 * A TCP/UDP socket server which does all the connection handling for you.
 * Specify a address and port and LSocketServer will do the rest for you.
 */

class LCL_EXPORT LSocketServer {
  LVCS_DECLARE;
 protected:
  LSocket4Inet * serverSocket;
  bool asyncConnections;

  void emitConnection (  LSocket4Inet * );

 public:

  LEmitter1 <LSocketServer&> startServer;  //!< Emitted when the server is started
  LEmitter1 <LSocketServer&> stopServer; //!< Emitted when the server is stopped
  LEmitter1 <LSocket4Inet&>   connection; //!< Emitted when the server accepted a new connection
  LEmitter2 <LBuffer&, LSockAddr4Inet&>   packet; //!< Emitted when the server reads a new UDP packet

  enum SocketOptions { reuseaddr = 100 };
  enum SocketDomain { LSS_AF_INET, LSS_AF_UNIX, LSS_AF_ISO, LSS_AF_NS, LSS_AF_IMP };
  enum SocketType { LSS_TYPE_STREAM, LSS_TYPE_DGRAM, LSS_TYPE_RAW, LSS_TYPE_SEQPACKET, LSS_TYPE_RDM };
  enum SocketProtocol { LSS_PROT_DEFAULT, LSS_PROT_IP, LSS_PROT_ICMP, LSS_PROT_IGMP, LSS_PROT_GGP, LSS_PROT_TCP, LSS_PROT_PUP, LSS_PROT_UDP, LSS_PROT_IDP };

  LSocketServer ();
  ~LSocketServer ();

  // socket slots
  void serverSocketRead ( LFileDesc * );

  void createSocket ( SocketOptions = reuseaddr, SocketDomain = LSS_AF_INET, SocketType = LSS_TYPE_STREAM, SocketProtocol = LSS_PROT_DEFAULT ); //!< Creates a new socket for the server. Must be called for initialization!
  void destroySocket (); //!< destroy the socket create with createSocket
  void setAsync ( bool = true ); //!< Sets the server to work in async mode. Creates a new thread for each connection

  void bind ( LSockAddr4Inet const & ); //!< Bind the server socket to an address
  void listen ( int = 5 ); //!< Set the socket to listen mode

  void start ();
  void stop ();

  bool valid () const;

  LSocket4Inet & getSocket (); //!< Get the server socket
  LSocket4Inet const & getSocket () const; //!< Get the server socket
};

#endif
