/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

#ifndef _LTRACE_H_
#define _LTRACE_H_

#include <lcl.h>
#include <lcl/lhash.h>
#include <lcl/lmutex.h>
#include <lcl/lstack.h>
#include <lcl/lheap.h>
#include <lcl/lutils.h>

/*
class LTracer;
class LTraceChannel;
*/

/*!
 * \defgroup Tracing Tracing
 */

class LCL_EXPORT LTraceMessage {
 protected:
  char * m_channel;
  char * m_msg;
  char * m_file;
  luint32 m_time;
  luint32 m_utime;
  luint32 m_line;
  luint32 m_pid;
  luint32 m_tid;
 public:
  LTraceMessage ();
  LTraceMessage ( LTraceMessage const & p1 );
  LTraceMessage ( char const * channel, char const * msg, char const * _file, luint32 line, luint32 _time, luint32 _utime, luint32 _pid, luint32 _tid );
  ~LTraceMessage ();

  char const * getChannel () const;
  char const * getMessage () const;
  char const * getFile () const;
  luint32 getTime() const;
  luint32 getUTime () const;
  luint32 getLine () const;
  luint32 getPID () const;
  luint32 getTID () const;
};

/*!
 * \class LTraceModule
 * \author Jens Reimann
 * \ingroup Tracing
 */

/*
class LCL_EXPORT LTraceModule {
  friend class LTracer;
 protected:  
  char * name;  

 public:
  LTraceModule ( const char * name );

};

typedef struct LTraceModuleInfo {
  LTraceModule * module;

  LTraceModuleInfo * next;
} LTraceModuleInfo;
*/

class LCL_EXPORT LTracerX;

class LCL_EXPORT LTraceOutput {
  friend class LCL_EXPORT LTracerX;
 protected:
  LSet<LString> filterList;
  LGuardMutex controlLock;
  bool isReg;

  void reg ();
  void unreg ();

  bool isMatch ( LTraceMessage const & );

 public:

  LTraceOutput ();
  virtual ~LTraceOutput ();

  void traceX ( LTraceMessage const & );
  virtual void trace ( LTraceMessage const & ) = 0;

  void addFilter ( LString const & );
  void removeFilter ( LString const & );

  LSet<LString> getFilters () const;

  virtual void setOptions ( LHash<LString,LString> const & vars ) = 0;
};

/*!
 * \class LTracer
 * \author Jens Reimann
 * \ingroup Tracing
 */

class LCL_EXPORT LTracerX {
  friend class __lcl_tracex_calc;
 protected:
  static LTracerX * theInstance;
  static LGuardMutex instanceLock;
  LTracerX ();

 public:
  static LTracerX * getInstance();

 protected:
  LHash<LString,bool> channelStates;
  LHash<LThreadID, bool> recursiveStates;
  LHash<LString, luint32> filterList;
  LHash<LString, LManagedPointer<LTraceOutput> > outputHash;
  LHash<LThreadID, LHash<LString, luint32> > stickiesHash;

  LGuardMutex controlLock;

  void calculateStates ();

  void setInTrace ( bool state );

  bool isMatch ( LString const & channel );

 public:

  bool enabled;
  bool inTrace () const;

  void addChannel ( LString const & );
  void addFilter ( LString const & );
  void removeFilter ( LString const & );
  bool channelActive ( LString const & );

  void trace ( LTraceMessage const & );

  void addOutput ( LString const & name, LManagedPointer<LTraceOutput> output );
  void removeOutput ( LString const & name );
  LSet<LString> getOutputs () const;
  LManagedPointer<LTraceOutput> getOutput ( LString const & );

};

/*
class LCL_EXPORT LTracer {
 protected: 
  LTraceModuleInfo * modules;
  LHash<LString,LString> moduleStates;

  static LTracer * theInstance;

  LTracer ();

 public:
  bool state;
  bool enable_all;

  void trace ( const char * module, const char * message );

  void addModule ( LTraceModule * );

  static LTracer * getInstance ();

  bool moduleState ( LString const & module );
  void enableModule ( LString const & module, bool state = true );
};
*/


/*!
 * \def LTRACE(mod,msg)
 *
 * The macro used for sending out a trace message
 *
 * \param mod the name of the trace channel
 * \param msg the message to send out on this channel
 */

/*!
 * \def LTRACE_FUNC(mod,name)
 *
 * The macro used for loggin the call of a function in the trace system.
 * 
 * \param mod the name of the module. The trace channel name will be mode".func" (this is depricated, the default channel is now "func" only)
 * \param name the function name (possibly with arguments). If the compiler supports it, LCL will let the compiler regenerate the function name.
 *
 * \code
void myFunc ( void )
{
  LTRACE_FUNC("myFunc", "void myFunc ( void )" );
  // This will send out the function name "void myFunc ( void )" including the stack depth
  // to the trace channel "myFunc.func"

  ... code ...

  // at the end of the function the same message as at the beginning of the function will be sent
  // but indicating that the function will be left
}
 * \endcode
 */


#ifdef HAVE_PRETTY_FUNCTION_MACRO
  #define L_FUNC_DESC __PRETTY_FUNCTION__
  #define L_FUNC_TRACE_DESC(d) __PRETTY_FUNCTION__
#else
  #define L_FUNC_DESC ""
  #define L_FUNC_TRACE_DESC(d) (d)
#endif

/*
#define L_NOTRACE

#ifdef L_NOTRACE
#define LTRACE(mod,msg)
#define LTRACE_FUNC(mod,name)
#else
#define LTRACE(mod,msg) if ( LTracer::getInstance()->state ) LTracer::getInstance()->trace((mod),(msg))
#define LTRACE_FUNC(mod,name)  LTraceFunc funcTrace ( L_FUNC_TRACE_DESC((name)), __FILE__, __LINE__ )
#endif
*/

#define LTRACEX_FUNC(name)  LTraceFunc funcTrace ( L_FUNC_TRACE_DESC((name)), __FILE__, __LINE__ )

#define LTRACEX(chan,msg) if ( LTracerX::getInstance()->enabled && (!LTracerX::getInstance()->inTrace()) && (LTracerX::getInstance()->channelActive((chan)))) LTracerX::getInstance()->trace( LTraceMessage ( (chan),(msg), __FILE__, __LINE__, ::time(0), 0, LGetProcessID(), LGetCurrentThreadID() ) )
// FIXME: msg will be generated twice!
#define LTRACEX_MSG(msg) if ( LTracerX::getInstance()->enabled && (!LTracerX::getInstance()->inTrace()) && (LTracerX::getInstance()->channelActive((msg).getChannel()))) LTracerX::getInstance()->trace( (msg) )

#define LTRACE(mod,msg) LTRACEX((mod),(msg))
#define LTRACE_FUNC(mod,msg) LTRACEX_FUNC((msg))

#include <lcl/ltracefunc.h>
#include <lcl/ltracechannel.h>

// Automatic Trace Channel info

typedef struct __LTraceChannelInfo {
  char * name;
  char * filter;
  char * desc;

  __LTraceChannelInfo * next;
} LTraceChannelInfo;

class LCL_EXPORT __lcl_traceChannelInfo {
 public:
  __lcl_traceChannelInfo ( char const * name, char const * filter, char const * desc  );
};

LCL_EXPORT void __lcl_DumpTraceChannels ( LOStream & out );

#define LTRACE_CHANNEL_DYNAMIC(name, var, filter, desc) __lcl_traceChannelInfo (var) ((name), (filter),(desc))
#define LTRACE_CHANNEL_DYN(name, var, filter, desc) LTRACE_CHANNEL_DYNAMIC((name),(var),(filter),(desc))
#define LTRACE_CHANNEL_STATIC(name, var, desc) LTRACE_CHANNEL_DYNAMIC((name),(var),NULL,(desc))

#endif
