/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

#include <lcl/lappcontrol_class.h>

#include <lcl/ltrace_output.h>


LManagedPointer<LApplicationControl_Class::Request> LApplicationControl_Class::fetchRequest ()
{
  return requestQueue.read ();
}

LApplicationControl_Class::LApplicationControl_Class () :
  requestSender ( requestQueue ),
  runnerFn ( &LApplicationControl_Class::xrunner, *this ),
  running ( false )
{
}

LApplicationControl_Class::~LApplicationControl_Class ()
{
}

void LApplicationControl_Class::queueRequest ( LApplicationControl_Base * control, LManagedPointer<LApplicationControl_Base::Request> request )
{
  requestSender.send ( new Request ( control, request ) );
}

void LApplicationControl_Class::xrunner ( void )
{
  running = true;
  runner ();
  running = false;
}

void LApplicationControl_Class::start ( void )
{
  if ( !running )
    {
      LThread tco ( &runnerFn );
      tco.start ();
    }
}

LApplicationControl_Class::Request::Request ( LApplicationControl_Base * control, LManagedPointer<LApplicationControl_Base::Request> request ) :
  myControl ( control ),
  myRequest ( request )
{
}

LApplicationControl_Base::Packet LApplicationControl_Class::Request::getData() const
{
  return myRequest->getPacket();
}

void LApplicationControl_Class::Request::sendReply ( LApplicationControl_Base::Packet const & packet )
{
  LApplicationControl_Base::Reply reply ( myRequest );
  reply.getPacket().setCommand ( packet.getCommand() );
  reply.getPacket().getVars() = packet.getVars();
  myControl->sendReply ( reply );
}

// LApplicationControl_DummyClass

void LApplicationControl_DummyClass::runner ( void )
{
  while ( true )
    {
      LManagedPointer<Request> request = fetchRequest ();

      LString cmd = request->getData().getVars().get ( "command", "" );      

      LApplicationControl_Base::Packet packet;

      LString reply;
      
      packet.setCommand( "OK" );
      if ( cmd == "ping" )
	{
	  reply = "pong";
	}
      else if ( cmd == "help" )
	{
	  reply = \
	    "\n\n" \
	    "LCL Application Dummy Class:\n" \
	    "============================\n" \
	    "  ping      -> get a pong\n" \
	    "  help      -> this information\n" \
	    "  suck      -> get no reply\n" \
	    "\n";
	    
	}
      else if ( cmd == "suck" )
	{
	  continue;
	}
      else
	{
	  packet.setCommand ( "error" );
	  packet.getVars().set ( "error", "Unkown command: '" + cmd + "'! Use 'help' to get some info!" );
	}

      if ( reply.length() )
	packet.getVars().set ( "reply", reply );

      request->sendReply ( packet );
    }
}

// LApplicationControl_TraceClass

void LApplicationControl_TraceClass::runner ( void )
{
  while ( true )
    {
      LManagedPointer<Request> request = fetchRequest ();

      LString cmd = request->getData().getVars().get ( "command", "" );

      LApplicationControl_Base::Packet packet;

      LString reply;
      
      packet.setCommand( "OK" );      
      if ( cmd == "help" )
	{
	  reply = \
	    "\n\n" \
	    "LCL Application Trace Class:\n" \
	    "============================\n" \
	    "  help      -> this information\n" \
	    "  enable    -> enable tracing module\n" \
	    "  disable   -> disable tracing module\n" \
	    "  show      -> show information\n" \
	    "     what:\n" \
	    "       outputs  -> all get list of outputs\n" \
	    "       filters  -> show filters (req. output to be set)\n" \
	    "     output:\n" \
	    "       the name of the output\n" \
	    "  add       -> add an output or filter\n" \
	    "     what:\n" \
	    "       output\n" \
	    "       filter\n" \
	    "     output:\n" \
	    "     filter:\n" \
	    "     type:\n" \
	    "  remove    -> remove an output or filter\n" \
	    "     what:\n" \
	    "       output\n" \
	    "       filter\n" \
	    "     output:\n" \
	    "     filter:\n" \
	    "     type:\n" \
	    "\n";
	    
	}
      else if ( cmd == "enable" )
	{
	  LTracerX::getInstance()->enabled = true;
	  reply = "enabled tracing";
	}
      else if ( cmd == "disable" )
	{
	  LTracerX::getInstance()->enabled = false;
	  reply = "disabled tracing";
	}
      else if ( cmd == "show" )
	{
	  LString what = request->getData().getVars().get ( "what", "" );
	  LString output = request->getData().getVars().get ( "output", "" );
	  if ( what == "outputs" )
	    {
	      LSet<LString> list = LTracerX::getInstance()->getOutputs ();
	      LDefaultSerializer ser;
	      ser << list;
	      reply = ser.getBuffer();
	    }
	  else if ( what == "filters" )
	    {
	      LManagedPointer<LTraceOutput> showOutput = LTracerX::getInstance() -> getOutput ( output );
	      if ( showOutput.isValid() )
		{
		  LSet<LString> list = showOutput->getFilters ();
		  LDefaultSerializer ser;
		  ser << list;
		  reply = ser.getBuffer();		  
		}
	      else
		{
		  packet.setCommand ( "error" );
		  packet.getVars().set ( "error", "No output " + output + " found!" );
		}
	    }
	  else
	    {
	      packet.setCommand ( "error" );
	      packet.getVars().set ( "error", "Unkown command: '" + cmd + "/" + what + "'! Use 'help' to get some info!" );
	    }
	}
      else if ( cmd == "add" )
	{
	  LString what = request->getData().getVars().get ( "what", "" );
	  LString output = request->getData().getVars().get ( "output", "" );
	  LString filter = request->getData().getVars().get ( "filter", "" );
	  LString type = request->getData().getVars().get ( "type", "" );
	  if ( what == "output" )
	    {
	      LManagedPointer<LTraceOutput> newOutput;
	      if ( type == "stream" )
		{
		  newOutput = new LTraceOutput_OStream;
		}
	      if ( newOutput.isValid() )
		{
		  LTracerX::getInstance() -> addOutput ( output, newOutput );
		  reply = "output " + output + " as type " + type + " added!";
		}
	      else
		{
		  packet.setCommand ( "error" );
		  packet.getVars().set ( "error", "unkown channel type " + type );
		}
	    }
	  else if ( what == "filter" )
	    {
	      LManagedPointer<LTraceOutput> changeOutput = LTracerX::getInstance() -> getOutput ( output );
	      if ( changeOutput.isValid() )
		{
		  changeOutput->addFilter ( filter );
		  reply = "filter " + filter + " added";
		}
	      else
		{
		  packet.setCommand ( "error" );
		  packet.getVars().set ( "error", "unkown output " + output );
		}
	    }
	  else
	    {
	      packet.setCommand ( "error" );
	      packet.getVars().set ( "error", "Unkown command: '" + cmd + "/" + what + "'! Use 'help' to get some info!" );
	    }
	}
      else if ( cmd == "remove" )	
	{
	  LString what = request->getData().getVars().get ( "what", "" );
	  LString output = request->getData().getVars().get ( "output", "" );
	  LString filter = request->getData().getVars().get ( "filter", "" );
	  if ( what == "output" )
	    {
	      LTracerX::getInstance() -> removeOutput ( output );
	      reply = "removed output " + output;
	    }
	  else if ( what == "filter" )
	    {
	      LManagedPointer<LTraceOutput> changeOutput = LTracerX::getInstance() -> getOutput ( output );
	      if ( changeOutput.isValid() )
		{
		  changeOutput->removeFilter ( filter );
		  reply = "removed filter " + filter + " from output " + output;
		}
	    }
	  else
	    {
	      packet.setCommand ( "error" );
	      packet.getVars().set ( "error", "Unkown command: '" + cmd + "/" + what + "'! Use 'help' to get some info!" );
	    }
	}
      else
	{
	  packet.setCommand ( "error" );
	  packet.getVars().set ( "error", "Unkown command: '" + cmd + "'! Use 'help' to get some info!" );
	}

      if ( reply.length() )
	packet.getVars().set ( "reply", reply );

      request->sendReply ( packet );
    }
}
