/* ****************************************************************************
   |      File: lbuffer.cc
   |      Contains: class LBuffer
   |      Created: 20.03.1998
   |      Last Changed: 26.03.1998
   |      Version: 0.0
   |         
   |      ToDo: + just started
   |
   ************************************************************************* */

/* ************************************************************************* *
 * Change log:                                                               *
 * -------------                                                             *
 *                                                                           *
 *   -> [v0.00] - 20.03.1998 - Created file and started coding               * 
 *   -> [v0.00] - 20.03.1998 - Ugh!! Found SOME bugs and eliminated them!!   *
 *   -> [v0.00] - 14.08.1998 - Changed some methods to const and added some  *
 *   -> [v0.00] - 14.08.1998 - method at class LTemplBuffer                  *
 * ************************************************************************* */


// **** LCL Includes

//#define DEBUG
#undef DEBUG
#include <lcl/lbuffer.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <iostream.h>
#include <fstream.h>
#include <lcl/ltrace.h>
#include <lcl/lmemory.h>
#include <lcl/llogsystem.h>


LBuffer::LBuffer () :
  m_ptr ( NULL ),
  m_size ( 0 )
{
}

LBuffer::LBuffer ( unsigned long _size )
{
  m_size = _size;
  m_ptr = lcl_malloc ( m_size, "LBuffer::LBuffer(luint32)" );

  erase ();
}

LBuffer::LBuffer ( unsigned long _size, void * buffer ) :
  m_ptr ( NULL ),
  m_size ( 0 )
{
  append ( buffer, _size );
}

LBuffer::LBuffer ( const LBuffer & p1 ) :
  m_ptr ( NULL ),
  m_size ( 0 )
{
  append ( p1 );
}

LBuffer::LBuffer ( const char * str ) :
  m_ptr ( NULL ),
  m_size ( 0 )
{
  append ( str );
}

LBuffer::LBuffer ( LString const & str ) :
  m_ptr ( NULL ),
  m_size ( 0 )
{
  append ( (char const*)str );
}

LBuffer::~LBuffer ()
{
  clear();
}

void const * LBuffer::getConstPtr () const
{
  return m_ptr;
}

unsigned long LBuffer::getSize () const
{
  return m_size;
}

const LBuffer & LBuffer::resize ( unsigned long _size )
{
  char *tmpptr;

  m_ptr = lcl_realloc ( m_ptr, _size, "LBuffer::resize" );

  if ( (m_ptr == NULL) && (_size != 0 ) )
    throw LOutOfMemoryException ( __FILE__, __LINE__ );

  if ( m_size < _size ) {
    tmpptr = (char*)m_ptr;
    memset ( tmpptr+m_size, 0, _size-m_size );    
  }

  m_size = _size;

  return *this;
}

const LBuffer & LBuffer::grow ( int i )
{
  if ( (m_size+i) < 0 )
    throw "Wrong Parameter";

  if ( (m_size+i) == 0 ) {
    clear();
  } else {
    resize ( m_size+i );
  }
  
  return *this;
}

void LBuffer::clear ()
{
  if ( m_ptr != NULL ) {
    lcl_free ( m_ptr ); 
    m_size = 0;
    m_ptr = NULL;
  }
}

void LBuffer::erase ()
{
  if ( m_ptr ) {
    memset ( m_ptr, 0, m_size );
  }
}

LBuffer::operator void*() 
{
  return m_ptr;
}

LBuffer::operator char*() 
{
  return (char*)m_ptr;
}


char LBuffer::operator [] ( int i ) const
{
  if ( i >= getSize() )
    throw 1;
  return ((char*)m_ptr)[i];
}

char LBuffer::operator [] ( unsigned long i ) const
{
  if ( i >= getSize() )
    throw 1;
  return ((char*)m_ptr)[i];
}

char & LBuffer::operator [] ( unsigned long i )
{
  if ( i >= getSize() )
    throw 1;
  return ((char*)m_ptr)[i];
}

bool LBuffer::operator == ( const LBuffer & p1 ) const {
  if ( m_size != p1.getSize() )
    return 0;

  return memcmp ( m_ptr, p1.m_ptr, m_size ) == 0;
}

LBuffer & LBuffer::operator =(const char* p1)
{
  clear ();
  append ( p1 );

  return *this;
}

LBuffer & LBuffer::operator =(const LBuffer & p1)
{
  clear();
  append ( p1 );

  return *this;
}

void LBuffer::operator+=(const LBuffer & p1 )
{
  append ( p1 );
}

void LBuffer::operator+=(char c)
{
  append ( (void*)&c, 1 );
}

LBuffer LBuffer::operator+(const LBuffer & p1 )
{
  LBuffer buffer = *this;
  buffer.append ( p1 );
  return buffer;
}

LBuffer & LBuffer::loadFile ( LString fname  )
{

  #ifdef _LCL_OPT_FOPEN_BIN_
  FILE * fp = fopen ( fname, "rb" );
  #else
  FILE * fp = fopen ( fname, "r" );
  #endif
  loadFile ( fp );
  if ( fp )
    fclose ( fp );

  return *this;
}

LBuffer & LBuffer::loadFile ( FILE * fp )
{
  char buffer[1024];
  unsigned long ammount;

  if ( fp == NULL ) {
    clear();
    return *this;
  }

  while ( !feof(fp ) ) {
    ammount = fread ( buffer, 1, 1024, fp );
    append ( (void*)buffer, ammount );
  }

  return *this;
}

LBuffer & LBuffer::loadFile ( istream & ins )
{
  LTRACE_FUNC("LBuffer", "loadFile ( istream & ");
  char buffer[1024];
  int pos=0;

  //  ins.seekg ( 0, ios::beg );

  while ( (buffer[pos] = ins.get()) != EOF ) {
    pos++;
    if ( pos == 1024 ) {
      append ( (void*)buffer, 1024 );
      pos = 0;
    }    
  }
  append ( (void*)buffer, pos );

  return *this;
}

void LBuffer::saveFile ( FILE * fp ) const
{
  fwrite ( (char*)m_ptr, m_size, 1, fp );
}

void LBuffer::saveFile ( LString fname ) const
{
  //  ofstream out_file ( fname, ios::out | ios::trunc );
  #ifdef _LCL_OPT_FOPEN_BIN_
  FILE * fp = fopen ( fname, "wb" );
  #else
  FILE * fp = fopen ( fname, "w" );
  #endif
  
  if ( !fp )
    {
      LLOG ( LLogSystem::ERR, "Could not open file: " + fname + " for writing" );
      return;
    }

  saveFile ( fp );

  fclose(fp);

  //  out_file.close();  
}

void LBuffer::saveFile ( ostream & outs ) const
{
  outs.write ( (char*)m_ptr, m_size );
}


// *************************************************************************
// *  Cut Functions  <<<----------
// *************************************************************************

LBuffer LBuffer::cutFrom ( unsigned long from ) const
{
  LBuffer newBuffer ( *this );
  newBuffer.cutFrom ( from );
  return newBuffer;
}

void LBuffer::cutFrom ( unsigned long from )
{
  if ( from == 0 )
    return;

  if ( from >= getSize() ) {
    clear ();
  } else {
    void * newptr = lcl_malloc ( getSize()-from, "LBuffer::cutFrom" );
    memcpy ( newptr, ((char*)m_ptr)+from, getSize()-from );
    void * tmpptr = m_ptr;
    m_ptr = newptr;
    m_size -= from;
    lcl_free ( tmpptr );
  }
}


// *************************************************************************
// *  Append Functions  <<<----------
// *************************************************************************

const LBuffer & LBuffer::append ( const LBuffer & p1 )
{
  return append ( p1.getConstPtr(), p1.getSize() );
}

const LBuffer & LBuffer::append ( void const * _ptr, unsigned long _size )
{
  unsigned long oldsize = m_size;
  char * tmpptr;

  grow ( _size );

  tmpptr = (char*)m_ptr;
  memcpy ( tmpptr+oldsize, _ptr, _size );

  return *this;
}

const LBuffer & LBuffer::append ( char const * str )
{
  append ( (void*)str, strlen ( str ) );
  return *this;
}

LBuffer & LBuffer::append ( char c )
{
  append ( (void*)&c, 1 );
  return *this;
}

LBuffer & LBuffer::append ( LString & str )
{
  append ( (void*)str.toChar(), str.length() );
  return *this;
}

void * LBuffer::getPtr ( void ) const
{
  return m_ptr;
}

void LBuffer::prepend ( const LBuffer & p1 )
{
  cout << "LBuffer prepend\n";

  char * tmpptr;

  if ( m_size == 0 ) {
    *this = p1;
  } else {
    if (p1.m_size != 0) {

      void * temp = lcl_malloc ( m_size + p1.m_size, "LBuffer::prepend" );
      if ( temp == NULL )
	throw LOutOfMemoryException ( __FILE__, __LINE__ );

      memcpy ( temp, p1.m_ptr, p1.m_size );
      tmpptr = (char*)temp;
      memcpy ( tmpptr+p1.m_size, m_ptr, m_size );
      m_size += p1.m_size;
      
    }
  }
    
}

LString LBuffer::toString () const
{
  LString str(*this);
  return str;
}

ostream & operator <<(ostream & os, LBuffer p1)
{
  os << p1.toString();
  return os;
}
