#include <lcl/lcolor.h>

LColor::LColor () :
  red ( 0 ),
  green ( 0 ),
  blue ( 0 ),
  alpha ( 0 ),
  valid ( true )
{
}

LColor::LColor ( unsigned char _red, unsigned char _green, unsigned char _blue, unsigned char _alpha ) :
  red ( _red << 24),
  green ( _green << 24),
  blue ( _blue << 24),
  alpha ( _alpha << 24),
  valid ( true )
{
}

LColor::LColor ( int _red, int _green, int _blue, int _alpha ) :
  red ( _red << 24),
  green ( _green << 24),
  blue ( _blue << 24),
  alpha ( _alpha << 24),
  valid ( true )
{
}

LColor::LColor ( unsigned long _red, unsigned long _green, unsigned long _blue, unsigned long _alpha ) :
  red ( _red ),
  green ( _green ),
  blue ( _blue ),
  alpha ( _alpha ),
  valid ( true )
{
}

LColor::LColor ( double _red, double _green, double _blue, double _alpha ) :
  red ( (_red>=1) ? (0xFFFFFFFF) : ((unsigned long)_red*0xFFFFFFFF) ),
  green ( (_green>=1) ? (0xFFFFFFFF) : ((unsigned long)_green*0xFFFFFFFF) ),
  blue ( (_blue>=1) ? (0xFFFFFFFF) : ((unsigned long)_blue*0xFFFFFFFF) ),
  alpha ( (_alpha>=1) ? (0xFFFFFFFF) : ((unsigned long)_alpha*0xFFFFFFFF) ),
  valid ( true )
{  
}

unsigned long LColor::getRed () const
{
  return red;
}

unsigned long LColor::getGreen () const
{
  return green;
}

unsigned long LColor::getBlue () const
{
  return blue;
}

unsigned long LColor::getAlpha () const
{
  return alpha;
}

unsigned char LColor::getRedAsChar () const
{
  return (unsigned char)(red >> 24);
}

unsigned char LColor::getGreenAsChar () const
{
  return (unsigned char)(green >> 24);
}

unsigned char LColor::getBlueAsChar () const
{
  return (unsigned char)(blue >> 24);
}

unsigned char LColor::getAlphaAsChar () const
{
  return (unsigned char)(alpha >> 24);
}

unsigned short LColor::getRedAsWord () const
{
  return (unsigned char)(red >> 16);
}

unsigned short LColor::getGreenAsWord () const
{
  return (unsigned char)(green >> 16);
}

unsigned short LColor::getBlueAsWord () const
{
  return (unsigned char)(blue >> 16);
}

unsigned short LColor::getAlphaAsWord () const
{
  return (unsigned char)(alpha >> 16);
}

LString LColor::asHexRGB8 () const
{
  char buffer[32];
  sprintf ( buffer, "#%02X%02X%02X", (int)getRedAsChar(), (int)getGreenAsChar(), (int)getBlueAsChar() );
  return buffer;
}

LString LColor::asHexRGBA8 () const
{
  char buffer[32];
  sprintf ( buffer, "#%02X%02X%02X%02X", (int)getRedAsChar(), (int)getGreenAsChar(), (int)getBlueAsChar(), (int)getAlphaAsChar() );
  return buffer;
}
  
LString LColor::asHexRGB16 () const
{
  char buffer[32];
  sprintf ( buffer, "#%04X%04X%04X", (int)getRedAsWord(), (int)getGreenAsWord(), (int)getBlueAsWord() );
  return buffer;
}

LString LColor::asHexRGBA16 () const
{
  char buffer[32];
  sprintf ( buffer, "#%04X%04X%04X%04X", (int)getRedAsWord(), (int)getGreenAsWord(), (int)getBlueAsWord(), (int)getAlphaAsWord() );
  return buffer;
}

void LColor::setRed ( unsigned char _red )
{
  red = (_red << 24);
}

void LColor::setGreen ( unsigned char _green )
{
  green = (_green << 24);
}

void LColor::setBlue ( unsigned char _blue )
{
  blue = (_blue << 24);
}

void LColor::setAlpha ( unsigned char _alpha )
{
  alpha = (_alpha << 24);
}

bool LColor::operator == ( LColor const & p1 ) const
{
  return (red==p1.red) && (green==p1.green) && (blue==p1.blue) && (alpha==p1.alpha);
}

bool LColor::operator != ( LColor const & p1 ) const
{
  return (red!=p1.red) || (green!=p1.green) || (blue!=p1.blue) || (alpha!=p1.alpha);
}

bool LColor::isValid () const
{
  return valid;
}

void LColor::setValid ( bool flag )
{
  valid = flag;
}

LColor LColor::asGray () const
{
  LColor color ( *this );
  color.toGray();
  return color;
}

void LColor::toGray ()
{
  unsigned long gray = (unsigned long)((red * 0.30) + (green * 0.59) + (blue * 0.11));
  red = green = blue = gray;  
}
