/* ****************************************************************************
   |      File: ldirectory.cc
   |      Contains: class LDirectory
   |      Created: 21.03.1999
   |      Last Changed: 21.03.1999
   |      Version: 0.0
   |         
   |      ToDo: + just started
   |
   ************************************************************************* */

/* ************************************************************************* *
 * Change log:                                                               *
 * -------------                                                             *
 *                                                                           *
 *   -> [v0.00] - 21.03.1999 - Created file and started coding               * 
 * ************************************************************************* */

#include <lcl/ldirectory.h>

#include <lcl/ltrace.h>
#include <lcl/lutils.h>

LDirectory::Entry::Entry ( LString const & _name, EntryType _type ) :
  name ( _name ),
  type ( _type )
{
}

LDirectory::Entry::EntryType LDirectory::Entry::getType ( ) const
{
  return type;
}

LString LDirectory::Entry::getName () const
{
  return name;
}

#ifdef _WIN32

LHeap<LDirectory::Entry> LDirectory::getListFull ( LString const & dirname )
{
  WIN32_FIND_DATA di;

  LHeap<Entry> entries;

  LString name = dirname;
  if ( dirname == "" )
    name = ".";

  HANDLE dir = FindFirstFile ( name+"\\*", &di );
  if ( dir == INVALID_HANDLE_VALUE )
    return entries;

  Entry::EntryType type;

  if ( di.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
    type = Entry::TYPE_DIR;
  else
    type = Entry::TYPE_FILE;
  entries.add ( Entry ( di.cFileName, type ) );
	
  while ( FindNextFile ( dir, &di ) )
    {      
      //      cout << FILE_ATTRIBUTE_DIRECTORY << " " << di.dwFileAttributes << " " << (di.dwFileAttributes | FILE_ATTRIBUTE_DIRECTORY)<< endl;
      if ( di.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
	type = Entry::TYPE_DIR;
      else
	type = Entry::TYPE_FILE;
      entries.add ( Entry ( di.cFileName, type ) );
      
    }

  FindClose ( dir );

  return entries;
}

#else

#include <sys/types.h>
#include <dirent.h>

LHeap<LDirectory::Entry> LDirectory::getListFull ( LString const & dirname )
{
  LHeap<Entry> list;

  struct dirent * di;
  DIR * dir = opendir ( dirname );

  struct stat fi;

  if ( !dir )
    return list;

  if ( dir ) {
    while ( (di = readdir ( dir )) ) {

      Entry::EntryType type;

      LString fname = dirname + "/" + di->d_name;
      if ( !stat ( fname, &fi ) )
	{
	  type = Entry::TYPE_FILE;

	  if ( S_ISDIR(fi.st_mode) )
	    type = Entry::TYPE_DIR;

	  list.add ( Entry ( di->d_name, type ) ); 
	}
      else
	{
	  LTRACE_PERROR();
	}

    }
    closedir ( dir );
  }
  return list;
}

#endif

LHeap<LString> LDirectory::getList ( LString const & dirname )
{
  LHeap<Entry> list = getListFull ( dirname );
  LHeap<LString> newList;

  LManagedPointer<LCursor> cursor = list.createCursor ();
  for ( cursor->setToFirst(); cursor->isValid(); cursor->setToNext() )
    {
      newList.add ( list.elementAt ( cursor ).getName () );
    }

  return newList;
}

LHeap<LString> LDirectory::getFileList ( LString const & dirname )
{
  LHeap<Entry> list = getListFull ( dirname );
  LHeap<LString> newList;

  LManagedPointer<LCursor> cursor = list.createCursor ();
  for ( cursor->setToFirst(); cursor->isValid(); cursor->setToNext() )
    {
      if ( list.elementAt ( cursor ).getType () == Entry::TYPE_FILE )
	newList.add ( list.elementAt ( cursor ).getName () );
    }

  return newList;
}

