/* ****************************************************************************
   |      File: lfdlist.cc
   |      Contains: class LFDList
   |      Created: 10.11.1998
   |      Last Changed: 10.11.1998
   |      Version: 0.0
   |         
   |      ToDo: + just started
   |
   ************************************************************************* */

/* ************************************************************************* *
 * Change log:                                                               *
 * -------------                                                             *
 *                                                                           *
 *   -> [v0.00] - 10.11.1998 - Created file and started coding               * 
 * ************************************************************************* */


// **** LCL Includes

#include <lcl.h>
#include <lcl/lfdlist.h>
#include <lcl/lpointer.h>
#include <assert.h>
#include <iostream.h>

LFDList::LFDList ()
{
  clear ();
}

LFDList::LFDList ( LFDList const & p1 )
{
  copyFrom ( p1 );
}

LFDList::LFDList ( fd_set const & p1 )
{
  copyFrom ( &p1 );
}

LFDList::LFDList ( fd_set const * p1 )
{
  copyFrom ( p1 );
}

void LFDList::clear ()
{
  m_fds.clear ();
}

void LFDList::copyFrom ( LFDList const & p1 )
{
  m_fds = p1.m_fds;
}

void LFDList::copyFrom ( fd_set const * p1 )
{
  assert ( p1 );
  clear ();
  addAllFrom ( p1 );
}

void LFDList::addAllFrom ( LFDList const & p1 )
{
  m_fds += p1.m_fds;
}

void LFDList::addAllFrom ( fd_set const * p1 )
{
  int i;

  assert ( p1 );

  for ( i=0; i<FD_SETSIZE; i++) {
    if ( FD_ISSET(i,p1) )
      set(i);
  }
}

void LFDList::set ( int fd )
{
  //  FD_SET ( fd, &m_fds );
  m_fds.add ( fd );
}

void LFDList::unset ( int fd )
{
  //  FD_CLR ( fd, &m_fds );
  m_fds.removeElement ( fd );
}

bool LFDList::isSet ( int fd ) const
{
  //return FD_ISSET ( fd, &m_fds );
  return m_fds.containsElement ( fd );
}

LFDList & LFDList::operator = ( LFDList const & p1 )
{
  clear ();
  copyFrom ( p1 );
  return *this;
}

LFDList LFDList::operator + ( LFDList const & p1 ) const
{
  LFDList new_fds ( *this );
  new_fds.addAllFrom ( p1 );
  return new_fds;
}

void LFDList::operator += ( LFDList const & p1)
{
  addAllFrom ( p1 );
}

LFDList::operator fd_set ( )
{
  //  return &m_fds;
  fd_set fds;
  LManagedPointer<LCursor> cursor = m_fds.createCursor();

  FD_ZERO ( &fds );

  for ( cursor->setToFirst(); cursor->isValid(); cursor->setToNext() ) {
    FD_SET ( m_fds.elementAt ( cursor ), &fds );
  }
  return fds;
}

int LFDList::highest () const
{
  int fd;
  int high = -1;
  LManagedPointer<LCursor> cursor = m_fds.createCursor ();
  
  for ( cursor->setToFirst(); cursor->isValid(); cursor->setToNext() ) {
    fd = m_fds.elementAt ( cursor );
    if ( fd > high )
      high = fd;
  }
  return high;
}

LSet<int> const & LFDList::getList () const
{
  return m_fds;
}

void LFDList::dump ( void ) const
{
  cout << "Dump of LFDList:\n";
  cout << "-----------------\n";

  LManagedPointer<LCursor> cursor = m_fds.createCursor ();
  for ( cursor->setToFirst(); cursor->isValid(); cursor->setToNext() ) {
    cout << "fd: " << m_fds.elementAt ( cursor ) << "\n";
  }

  cout << "-----------------\n";
  cout << "Highest fd: " << highest() << "\n";
  cout << "-----------------\n";
}
