#include <lcl/lhttp.h>
#include <lcl/lregex.h>
#include <lcl/lhashiter.h>
#include <lcl/ltrace.h>

LVCS_INIT(LHTTPBase);
LVCS_INIT(LHTTPRequest);

LHash<LString,LString> & LHTTPBase::getVars ()
{
  return vars;
}

void LHTTPBase::setContentType ( LString const & type )
{
  vars.set ( "Content-Type", type );
}

void LHTTPBase::setContentType ( LString const & p1, LString const & p2 )
{
  setContentType ( p1 + "/" + p2 );
}

LString LHTTPBase::getContentType ( ) const
{
  return vars.get ( "Content-Type", "" );
}

// LHTTPRequest

LHTTPRequest::LHTTPRequest ( LString const & _uri, LString const & _method,  LString const & _version )
{
  setMethod ( _method );
  setURI ( _uri );
  setVersion ( _version );
}


LHTTPRequest::LHTTPRequest ( LIStream & istream )
{
  readFromStream ( istream );
}

LString LHTTPRequest::getMethod () const
{
  return method;
}

LString LHTTPRequest::getURI () const
{
  return uri;
}

LString LHTTPRequest::getVersion () const
{
  return version;
}

void LHTTPRequest::setMethod ( LString const & _method )
{
  method = _method;
}

void LHTTPRequest::setVersion ( LString const & _version )
{
  version = _version;
}

void LHTTPRequest::setURI ( LString const & _uri )
{
  uri = _uri;
}

bool LHTTPRequest::readFromStream ( LIStream & istream )
{

  vars.clear ();

  LString line;

  line = istream.gets ();
  LRegEx exp ( "([a-zA-Z0-9]*) (.*) ([a-zA-Z0-9/\\.]*)" );

  if ( !exp.exec ( line ) ) {
    return false;
  }

  //  cout << "Valid HTTP 1.0+ header" << endl;
  //  cout << exp.match(1) << "-" << exp.match(2) << "-" << exp.match(3) << endl;
  
  setMethod ( exp.match(1) );
  setURI ( exp.match(2) );
  setVersion ( exp.match(3) );

  LRegEx exp2 ( "([a-zA-Z0-9\\-]*): (.*)" );

  line = istream.gets ();

  while ( line.length() ) {
    if ( exp2.exec ( line ) ) {
      vars.set ( exp2.match(1), exp2.match(2) );
    }
    line = istream.gets ();
  }

  return true;

}


/*
 * *****************************************************************************************************************************************
 *   LHTTPReponse
 *        Response to a HTTP Request
 * *****************************************************************************************************************************************
 */


class SendVarsIter : public LHashIterator <LString, LString> {
public:
  LOStream * ostream;
  void applyTo ( LString & key, LString & value )
    {
      (*ostream) << key << ": " << value << endl;
    }
};



LHTTPResponse::LHTTPResponse ( long _status, LString const & _reason, LString const & _version )
{
  setStatus ( _status );
  setReason ( _reason );
  setVersion ( _version );
}

LHTTPResponse::LHTTPResponse ( LIStream & istream )
{
  readFromStream ( istream );
}


long LHTTPResponse::getStatus () const
{
  return status;
}

LString LHTTPResponse::getReason () const
{
  return reason;
}

LString LHTTPResponse::getVersion () const
{
  return version;
}

void LHTTPResponse::setStatus ( long _status )
{
  status = _status;
}

void LHTTPResponse::setReason ( LString const & _reason )
{
  reason = _reason;
}

void LHTTPResponse::setVersion ( LString const & _version )
{
  version = _version;
}

bool LHTTPResponse::readFromStream ( LIStream & )
{ 
  return false;
}

bool LHTTPResponse::sendToStream ( LOStream & ostream )
{
  ostream << version << " " << status << " " << reason << endl;

  LTRACE ( "LHTTPResponse", "Sending " + LString(vars.entries()) + " entries" );

  SendVarsIter svi;
  svi.ostream = &ostream;
  vars.allElementsDo ( svi );
  
  ostream << endl;

  return true;
}

