#include <lcl/lhttpd.h>

#include <lcl/lstreamproxy.h>
#include <lcl/lstrstream.h>
#include <lcl/lutils.h>

#include <time.h>

/*
#include <unistd.h>
*/

LVCS_INIT(LHTTPDaemon);

long LHTTPConnection::timeoutValue = 30;

LHTTPDaemon::LHTTPDaemon ( ) :
  checkerFn ( &LHTTPDaemon::checker, *this )
{
  lout << "LHTTP Daemon v0.1 coming up" << endl;  

  LThread thread ( &checkerFn );
  thread.start ();

}

LHTTPDaemon::~LHTTPDaemon ()
{
  lout << "LHTTP Daemon shutdown!" << endl;
}

void LHTTPDaemon::checker ( )
{
  lout << "LHTTPDaemon::checker running..." << endl << flush;

  LCursor * cursor;
  LHTTPConnection * conn;
  LSet<LHTTPConnection*> removeConns;

  while ( true ) {
    lcl_usleep ( 10000 );
    cursor = connectionList.createCursor ();
    for ( cursor->setToFirst(); cursor->isValid(); cursor->setToNext () ) {
      conn = connectionList.elementAt ( cursor );

      if ( conn->lost () ) {
	lout << "Connection lost ... removing!" << endl << flush;
	removeConns.add ( conn );
      }
      if ( conn->timeout () ) {
	lout << "Connection timed out ... removing!" << endl << flush;
	removeConns.add ( conn );
      }
      if ( conn->complete () ) {
	lout << "Request complete!" << endl << flush;
	processComplete ( conn );
	removeConns.add ( conn );
      }

    }   
    delete cursor;

    cursor = removeConns.createCursor ();
    for ( cursor->setToFirst(); cursor->isValid(); cursor->setToNext () ) {
      lout << "Deleting element" << endl;
      conn = removeConns.elementAt ( cursor );
      connectionList.removeElement ( conn );
      delete conn;
    }
    removeConns.clear ();
    delete cursor;
  }

}

void LHTTPDaemon::processComplete ( LHTTPConnection * conn )
{
  LStrStream stream ( conn->getBuffer() );
  LHTTPRequest _request ( stream );

  request.emit ( _request, &conn->getSocket() );
}

void LHTTPDaemon::connection ( LSocket4Inet & socket )
{
  lout << "New HTTP Connection..." << endl;

  LHTTPConnection * conn = new LHTTPConnection ( socket );

  connectionList.add ( conn );
}


LHTTPConnection::LHTTPConnection ( LSocket4Inet const & _socket ) : 
  socket ( _socket ),
  lastData ( time(0) ),
  isLost ( false ),
  isComplete ( false ),
  cr ( false )
{
  lout << "LHTTPConnection::LHTTPConnection ( LSocket4Inet const & )" << endl;

  #ifndef _WIN32
  connect_emitter ( &LHTTPConnection::readEvent, *this, socket.readEvent );
  #else
  connect_emitter_1 ( &LHTTPConnection::readEvent, *this, socket.readEvent );
  #endif
  socket.setEventMask ( LFileDesc::eventRead );

  socket.reg ( LSelectLoop::getInstance () );

}

void LHTTPConnection::readEvent ( LFileDesc * fd )
{
  LTRACE_FUNC ( "LHTTPConnection", "readEvent" );

  if ( fd != &socket )
    throw;

  char buffer [1025];
  int len = socket.read ( buffer, 1024 );  

  lout << "Read " << (long)len << " bytes of data!" << endl << flush;

  if ( len == 0 ) {

    lout << "Connection lost!" << endl;
    socket.unreg ( LSelectLoop::getInstance () );
    isLost = true;

  } else {

    lastData = time(0);
    long i;

    for ( i = 0; i < len; i++ ) {

      switch ( buffer[i] ) {
      case '\r':
	break;

      case '\n':
	if ( cr ) {
	  isComplete = true;
	  socket.unreg ( LSelectLoop::getInstance () );
	} else
	  cr = true;
	break;

      default:
	cr = false;
	break;
      }

    }

    buffer[i] = 0;
    string_buffer += buffer;

  }

}

bool LHTTPConnection::complete () const
{
  return isComplete;
}

bool LHTTPConnection::timeout () const
{
  return difftime ( time(0), lastData ) > timeoutValue;
}

bool LHTTPConnection::lost () const
{
  return isLost;
}

LString LHTTPConnection::getBuffer () const
{
  return string_buffer;
}

LSocket4Inet & LHTTPConnection::getSocket ()
{
  return socket;
}
