#include <lcl/lhttpdir.h>

#include <lcl/lstreamproxy.h>
#include <lcl/lregex.h>
#include <lcl/lthread.h>

//#include <pthread.h>

LVCS_INIT(LHTTPDirector);
LVCS_INIT(LHTTPProcessor);

class AsyncProcStruct : public LThreadFn {
protected:
  LHTTPProcessor * proc;
  LHTTPRequest request;
  LSocket4Inet socket;

public:

  AsyncProcStruct ( LHTTPRequest _request, LSocket4Inet const & _socket, LHTTPProcessor * _proc ) :
    proc ( _proc ),
    request ( _request ),
    socket ( _socket )
    {
    }
  virtual void run () 
  {
    deleteMe = true;

    proc->process ( request, socket );    
  }
};

/*
pthread_attr_t thread_attrs;
bool thread_attrs_init = false;
*/
/*

void * asyncRunner ( void * ptr )
{
  AsyncProcStruct * aps = (AsyncProcStruct*)ptr;
  if ( !aps )
    return NULL;
  aps->proc->process ( (aps->request), aps->socket );
  delete  aps ;
  return NULL;
}
*/

void LHTTPProcessor::Info::setName ( LString const & _name )
{
  name = _name;
}

void LHTTPProcessor::Info::setDescription ( LString const & _desc )
{
  description = _desc;
}

void LHTTPProcessor::Info::setAuthor ( LString const & _author )
{
  author = _author;
}

void LHTTPProcessor::Info::setVersion ( LString const & _version )
{
  version = _version;
}

void LHTTPProcessor::Info::setEntryPoint ( LString const & _entryPoint )
{
  entryPoint = _entryPoint;
}

LString LHTTPProcessor::Info::getName () const
{
  return name;
}

LString LHTTPProcessor::Info::getDescription () const
{
  return description;
}

LString LHTTPProcessor::Info::getAuthor () const
{
  return author;
}

LString LHTTPProcessor::Info::getVersion () const
{
  return version;
}

LString LHTTPProcessor::Info::getEntryPoint () const
{
  return entryPoint;
}


LHTTPDirector::LHTTPDirector ()
{
  lout << "Created new LHTTPDirector!" << endl;
}

void LHTTPDirector::processRequest ( LHTTPRequest request, LSocket4Inet * socket )
{
  LHTTPProcessor * proc;

  LManagedPointer<LCursor> cursor = procList.createCursor ();

  for ( cursor->setToFirst () ; cursor->isValid() ; cursor->setToNext () ) {
    proc = procList.elementAt ( cursor );    
    if ( proc->match ( request.getURI() ) ) {
      proc->preProcess ( request, *socket );
      return;
    }
  }
  errorProc->preProcess ( request, *socket );
}

void LHTTPDirector::addProcessor ( LString const & exp, LHTTPProcessor & proc )
{
  proc.setExpression ( exp );
  procList.add ( &proc );
}

void LHTTPDirector::removeProcessor ( LString const & exp )
{
  procList.removeElementWithKey ( exp );
}

void LHTTPDirector::setErrorProcessor ( LHTTPProcessor & _errorProc )
{
  errorProc = &_errorProc;
}

LHeap<LHTTPProcessor::Info> LHTTPDirector::getInfoList ( bool skipDummies ) const
{
  LHeap<LHTTPProcessor::Info> list;

  LManagedPointer<LCursor> cursor = procList.createCursor ();
  for ( cursor->setToFirst(); cursor->isValid(); cursor->setToNext() )
    {
      LHTTPProcessor::Info info = procList.elementAt ( cursor )-> getInformation(); 
      if ( !skipDummies || (info.getName() != "[unkown]" ) )
	list.add ( info );
    }

  return list;
}

// LHTTPProcessor

LHTTPProcessor::LHTTPProcessor () :
  async ( false )
{
}

void LHTTPProcessor::setExpression ( LString const & _exp )
{
  expression = _exp;
}

LString LHTTPProcessor::getExpression ( ) const
{
  return expression;
}

bool LHTTPProcessor::match ( LString const & uri ) const
{
  //  lout << expression << " <-> " << uri << endl;
  LRegEx exp ( expression );
  return exp.exec ( uri );
}

void LHTTPProcessor::preProcess ( LHTTPRequest request, LSocket4Inet & socket )
{
  if ( async ) {
    //    pthread_t tid;    
    AsyncProcStruct * aps = new AsyncProcStruct ( request, socket, this );
    LThread tco ( aps );
    tco.start ();
    //    pthread_create ( &tid, &thread_attrs, asyncRunner, aps );
  } else {
    process ( request, socket );
  }
}

void LHTTPProcessor::setAsync ( bool flag )
{
  async = flag;
}

LHTTPProcessor::Info LHTTPProcessor::getInformation () const
{
  Info info;

  info.setName ( "[unkown]" );
  info.setAuthor ( "[unkown]" );
  info.setVersion ( "[unkown]" );
  info.setDescription ( "[unkown]" );
  info.setEntryPoint ( "" );

  return info;
}

LCL_EXPORT LString key ( LHTTPProcessor * p1 )
{
  return p1->expression;
}

void LHTTPXProcessor::process ( LHTTPRequest req, LSocket4Inet & socket )
{
  try
    {
      saveProcess ( req, socket );
    }
  catch ( ... )
    {
      LTRACE ( "LHTTPXProcessor", "Exception caught" );
    }
}
