
#include <lcl/lcl_config.h>

#ifdef HAVE_LIBJPEG

#include <lcl/lstreamproxy.h>

#include <lcl/limage.jpeg.h>

#include <sys/types.h>
#include <stdio.h>

#include <jpeglib.h>

class JPEG_InputSource {
protected:

public:

  LIStream & stream;
  LBuffer buffer;
  unsigned long pos;

  JPEG_InputSource ( LIStream & _stream ) :
    stream ( _stream )
  {    
  }
};

void init_source ( j_decompress_ptr cinfo )
{
  //  lerr << "Init Source" << endl << flush;
  JPEG_InputSource * src = (JPEG_InputSource*)cinfo->client_data;
  src->pos = 0;
  cinfo->src->next_input_byte = (const JOCTET *)src->buffer.getPtr();
  cinfo->src->bytes_in_buffer = 0;
}

void term_source ( j_decompress_ptr cinfo )
{
  //  lerr << "Term Source" << endl << flush;
  JPEG_InputSource * src = (JPEG_InputSource*)cinfo->client_data;
}

boolean fill_input_buffer ( j_decompress_ptr cinfo )
{
  //  lerr << "Fill Source" << endl << flush;
  JPEG_InputSource * src = (JPEG_InputSource*)cinfo->client_data;

  char buffer[4096];
  unsigned long num = src->stream.read ( (void*)buffer, sizeof(buffer) );
  src->buffer.append ( (void*)buffer, num );
  src->pos += num;
  cinfo->src->bytes_in_buffer += num;
  cinfo->src->next_input_byte = (const JOCTET *) ((char*)src->buffer.getPtr() + (src->pos-cinfo->src->bytes_in_buffer) );

  //  lerr << "Pos: " << (src->pos-num) << " Size: " << src->buffer.getSize() << " In: " << (long)cinfo->src->bytes_in_buffer;
  //  lerr << " Ptr: " << (unsigned long)src->buffer.getPtr() << " posptr: " << (unsigned long)((char*)src->buffer.getPtr() + (src->pos-cinfo->src->bytes_in_buffer));
  //  lerr << endl << flush;

  return TRUE;
}

void skip_input_data (j_decompress_ptr cinfo, long num_bytes)
{
  //  lerr << "Skip Source" << endl << flush;
  if ( num_bytes <= 0 )
    return;

  JPEG_InputSource * src = (JPEG_InputSource*)cinfo->client_data;
  if ( num_bytes <= cinfo->src->bytes_in_buffer ) {
    //    lerr << "Small Skip!" << endl << flush;
    cinfo->src->bytes_in_buffer -= num_bytes;
    cinfo->src->next_input_byte = (const JOCTET *) ((char*)cinfo->src->next_input_byte + num_bytes);
  } else {
    fill_input_buffer ( cinfo );
    //    lerr << "Large Skip!" << endl << flush;
    cinfo->src->bytes_in_buffer -= num_bytes;
    cinfo->src->next_input_byte = (const JOCTET *) ((char*)cinfo->src->next_input_byte + num_bytes);
  }
}

bool LImageReader_JPEG::impl_read ( LImage  & image, LIStream & stream ) const
{

  struct jpeg_decompress_struct cinfo;
  struct jpeg_error_mgr jerr;  

  struct jpeg_source_mgr jin;

  jpeg_create_decompress(&cinfo);

  // Setting up input manager...

  memset ( &jin, 0, sizeof(jin) );  
  cinfo.client_data = (void*)new JPEG_InputSource ( stream );
  jin.init_source = init_source;
  jin.term_source = term_source;
  jin.fill_input_buffer = fill_input_buffer;
  jin.skip_input_data = skip_input_data;
  jin.resync_to_restart = jpeg_resync_to_restart;

  cinfo.err = jpeg_std_error(&jerr);
  cinfo.src = &jin;


  // Start reading...

  jpeg_read_header(&cinfo, TRUE);
  
  lerr << (long)cinfo.image_width << "x" << (long)cinfo.image_height << " with " << (long)cinfo.out_color_components<< endl << flush;

  long cx = (long)cinfo.image_width;
  long cy = (long)cinfo.image_height;

  int i = 0;

  LBuffer data ;

  jpeg_start_decompress(&cinfo);

  long bpp = (long)cinfo.output_components;

  image.setBitsPerPixel ( bpp*8 );
  image.setSize ( cx, cy );

  lerr << (long)cinfo.image_width << "x" << (long)cinfo.image_height << " with " << (long)cinfo.output_components<< endl << flush;

  JSAMPARRAY buffer = (*cinfo.mem->alloc_sarray)((j_common_ptr) &cinfo, JPOOL_IMAGE, cx*bpp, 1);

  // ** READ  

  while ( i<cy ) {
    i += jpeg_read_scanlines ( &cinfo, buffer, 1 );
    data.append ( buffer[0], bpp*cx );
  }

  // Set the bits in LImage

  image.setBits ( data );
   

  delete (JPEG_InputSource*)cinfo.client_data;

  jpeg_destroy_decompress(&cinfo);


  return true;
}

#endif /* HAVE_LIBJPEG */


