#include <lcl.h>
#include <lcl/ltrace.h>

#include <string.h>
#include <lcl/liostream.h>
#include <stdio.h>

//LVCS_INIT(LStreamException);
LVCS_INIT(LErrorStreamException);
LVCS_INIT(LEndOfStreamException);
LVCS_INIT(LIStream);
LVCS_INIT(LOStream);
LVCS_INIT(LIOStream);

LOStream & endl ( LOStream & stream )
{
  stream << '\n';
  return stream;
}

LOStream & flush ( LOStream & stream )
{
  stream.flush ();
  return stream;
}

LStreamOException::LStreamOException ( int _err, LOStream &  _stream ) :
  LErrorStreamException ( _err ),
  stream ( _stream )
{  
}

LEndOfStreamException::LEndOfStreamException ( LIStream & _stream, unsigned long _nData) :
  stream ( _stream ),
  nData ( _nData )
{
  _stream.EOS = true;
}

LErrorStreamException::LErrorStreamException ( int _err ) :
  err ( _err )
{
}

LIStream::LIStream () :
  EOS ( false )
{
}

int LIStream::read ( void * _ptr, unsigned long len )
{
  unsigned long int i;
  int rc, cnt = 0;
  char * ptr = (char*)_ptr;
  
  for ( i = 0; i< len; i++ ) {
    rc = read ( ptr[i] );
    LTRACE ( "LIStream", "read -> " + LString(rc) );
    if ( rc == -1 )
      //      return -1;
      throw LErrorStreamException ( rc );
    if ( rc == 0 )
      throw LEndOfStreamException ( *this, cnt );

    cnt += rc;
  }
  return cnt;
}

bool LIStream::eos () const
{
  return EOS;
}

LIStream & LIStream::operator >> ( char & c )
{
  read ( c );
  return *this;
}

LString LIStream::gets ( LString const & delims, LString const & ignore )
{
  char c;
  LString buffer;
  
  //  LTRACE ( "LIStream", "pre-read" );
  read ( c );
  //  LTRACE ( "LIStream", "post-read" );
  while ( !delims.contains ( c ) ) {
    if ( !ignore.contains ( c ) )
      buffer += c;
    //    LTRACE ( "LIStream", "pre-read" );
    read ( c );
    //    LTRACE ( "LIStream", "post-read" );
  }
  return buffer;
}

int LOStream::write ( void const * _ptr, unsigned long len )
{
  unsigned long int i;
  int rc, cnt = 0;
  char * ptr = (char*)_ptr;
      
  for ( i = 0; i< len; i++ ) {
    rc = write ( ptr[i] );
    if ( rc == -1 )
      return -1;
    if ( rc == 0 )
      return cnt;
    cnt += rc;
  }
  return cnt;
}

LOStream & LOStream::operator << ( LOStream & (*func)(LOStream&) )
{
  return func ( *this );
}

LOStream & LOStream::operator << ( char const * str )
{
  long len = strlen ( str );
  if ( !len )
    return *this;

  int rc = write ( str, len );

  if ( ( rc == 0 )  || ( rc == -1 ) )
    throw LStreamOException ( getLastError(), *this );

  return *this;
}

LOStream & LOStream::operator << ( char c )
{
  int rc = write ( c );

  if ( ( rc == 0 )  || ( rc == -1 ) )
    throw LStreamOException ( getLastError(), *this );

  return *this;
}

LOStream & LOStream::operator << ( int i )
{
  char buffer[64];
  sprintf ( buffer, "%i", i );
  return *this<<buffer;
}

LOStream & LOStream::operator << ( unsigned int i )
{
  char buffer[64];
  sprintf ( buffer, "%u", i );
  return *this<<buffer;
}

LOStream & LOStream::operator << ( long l )
{
  char buffer[64];
  sprintf ( buffer, "%li", l );
  return *this<<buffer;
}

LOStream & LOStream::operator << ( unsigned long ul )
{
  char buffer[64];
  sprintf ( buffer, "%lu", ul );
  return *this<<buffer;
}

LOStream & LOStream::operator << ( short i )
{
  char buffer[64];
  sprintf ( buffer, "%i", i );
  return *this<<buffer;
}

LOStream & LOStream::operator << ( unsigned short i )
{
  char buffer[64];
  sprintf ( buffer, "%u", i );
  return *this<<buffer;
}

LOStream & LOStream::operator << ( double d )
{
  char buffer[64];
  sprintf ( buffer, "%lf", d );
  return *this<<buffer;
}

void LOStream::flush ()
{
}
