#include <lcl/llibrary.h>

#include <lcl.h>

#include <lcl/ltrace.h>

LLibrary::LLibrary () :
  handle ( LLIBRARY_INVALID )
{
}

/*!
 * \param file The filename to the library
 *
 * Calls LLibrary::load to load the library
 */
LLibrary::LLibrary ( LString const & file ) :
  handle ( LLIBRARY_INVALID )
{
  load ( file );
}

LLibrary::~LLibrary ()
{
  unload ( );
}

/*!
 * \return true if the library could be loaded
 */
bool LLibrary::isValid() const
{
  return handle != LLIBRARY_INVALID;
}

#ifdef PURE_WIN32
void * LLibrary::getProc ( LString const & name )
{
  if ( !isValid() )
    return NULL;

  return (void*)GetProcAddress ( handle, name );
}

void LLibrary::load ( LString const & name )
{
  unload ();
  handle = LoadLibrary ( name );
}

void LLibrary::unload ()
{
  if ( !isValid() )
    return;

  FreeLibrary ( handle );
}
#else

#ifndef RTLD_LOCAL
#define RTLD_LOCAL 0
#endif

/*!
 * \return the pointer to the symbol
 */
void * LLibrary::getProc ( LString const & name )
{
  if ( !isValid() )
    return NULL;

  return (void*)dlsym ( handle, name );
}

/*!
 * \param name the filename of the library
 *
 * Depending on the system configuration and the OS several lookup rules of the file
 * may be applied.
 */
void LLibrary::load ( LString const & name )
{
  unload ();
  handle = dlopen ( name, RTLD_NOW | RTLD_LOCAL );
  if ( handle == LLIBRARY_INVALID )
    {
      LTRACE ("perror", LString::perror() );
      LTRACE ("LLibrary", LString("dlerror:" ) + dlerror() );
    }
}

/*!
 * Unload the library
 */
void LLibrary::unload ()
{
  if ( !isValid() )
    return;

  dlclose ( handle );
}

#endif

