#include <lcl/lrevision.h>

#include <lcl/lstreamproxy.h>

static LModuleRecord * __rootModule = NULL;

LModuleRecord * addModule ( char const * name )
{
  LModuleRecord * newModule = NULL;
  LModuleRecord * currentModule = NULL;  

  if ( !__rootModule )
    {
      __rootModule = new LModuleRecord;
      newModule = __rootModule;
    }
  else
    {
      newModule = new LModuleRecord;
      currentModule = __rootModule;

      // Go to last module
      while ( currentModule->nextModule )
	currentModule = currentModule->nextModule;

      // and append the new one
      currentModule->nextModule = newModule;

    }

  //  newModule = new LModuleRecord;
  newModule->firstRevision = NULL;
  newModule->firstFeature = NULL;
  newModule->nextModule = NULL;
  newModule->name = strdup ( name );

  return newModule;
}

LModuleRecord * getModule ( char const * name )
{
  LModuleRecord * currentModule = NULL;

  if ( __rootModule )
    {

      currentModule = __rootModule;
      while ( currentModule )
	{
	  if ( strcmp ( currentModule->name, name ) == 0 )
	    return currentModule;
	  currentModule = currentModule->nextModule;
	}

      // not found, so add it!
      return addModule ( name );
    }
  else
    {
      // no root entry ... add it
      return addModule ( name );
    }
}

void addFeature ( LModuleRecord * module,  char const * name )
{
  LFeatureRecord * current, *temp,*prev;

  prev = NULL;
  current = module->firstFeature;

  while ( current )
    {
      if ( strcmp ( current->name, name ) == 0 )
	return;
      prev = current;
      current = current->nextFeature;
    }

  temp = new LFeatureRecord;
  if ( prev )
    {
      prev->nextFeature = temp;
    }
  else
    {
      module->firstFeature = temp;
    }

  temp->nextFeature = NULL;
  temp->name = strdup ( name );
}

LFeature::LFeature ( char const * module, char const * name )
{
  LModuleRecord * mod = getModule ( module );
  addFeature ( mod, name );
}

void dumpFeatures ( LModuleRecord * module )
{
  LFeatureRecord * current = module->firstFeature;
  while ( current )
    {
      lout << "\tFeature: " << current->name << endl;
      current = current->nextFeature;      
    }
}


void addRevision ( LModuleRecord * module, char const * version, char const * description )
{
  LRevisionRecord * current, *temp,*prev;

  prev = NULL;
  current = module->firstRevision;

  while ( current )
    {
      if ( strcmp ( current->version, version ) == 0 )
	return;
      prev = current;
      current = current->nextRevision;
    }

  temp = new LRevisionRecord;
  if ( prev )
    {
      prev->nextRevision = temp;
    }
  else
    {
      module->firstRevision = temp;
    }

  temp->nextRevision = NULL;
  temp->version = strdup ( version );
  temp->description = strdup ( description );
}


LRevision::LRevision ( char const * module, char const * version, char const * description )
{
  LModuleRecord * mod = getModule ( module );
  addRevision ( mod, version, description );
}

void dumpRevisions( LModuleRecord * module )
{
  LRevisionRecord * current = module->firstRevision;
  while ( current )
    {
      lout << "\tVersion: " << current->version << endl << "\t\t" << current->description << endl;
      current = current->nextRevision;      
    }
}


void LModuleInfo::dump ( LOStream & os )
{
  LModuleRecord * current = __rootModule;

  os << "---------- Dump Modules ----------" << endl;

  while ( current )
    {
      os << "Module: " << current->name << endl;
      dumpFeatures ( current );
      dumpRevisions ( current );
      current = current->nextModule;
    }
  os << "---------- End of Dump -----------" << endl;
  os << flush;
}

LFEATURE("VersionControl", vc_feat, "Feature Identification System" );
LFEATURE("VersionControl", vc_rev, "Revision Control System" );

LREVISION("VersionControl", vc_feat20, "2.0", "Enhanced release with LRevision and LFeature" );
