/* ****************************************************************************
   |      File: ltime.cc
   |      Contains: class LTime
   |      Created: 29.07.1998
   |      Last Changed: 29.07.1998
   |      Version: 0.0
   |         
   |      ToDo: + just started
   |
   ************************************************************************* */

/* ************************************************************************* *
 * Change log:                                                               *
 * -------------                                                             *
 *                                                                           *
 *   -> [v0.00] - 05.03.1998 - Created file and started coding               * 
 * ************************************************************************* */

#include <lcl.h>
#include <lcl/ltime.h>

#ifdef HAVE_TIME_H
#include <time.h>
#endif

LTime::LTime () :
  m_time ( 0 )
{
}

LTime::LTime ( LTime const & p1 ) :
  m_time ( p1.m_time )
{
}

LTime::LTime ( luint32 _time ) :
  m_time ( _time )
{
}

LTime::LTime ( luint8 hours, luint8 mins, luint8 secs, luint16 msecs )
{
  set ( hours, mins, secs, msecs );
}

void LTime::set ( LTime const & p1 )
{
  m_time = p1.m_time;
}

void LTime::set ( luint8 hours, luint8 mins, luint8 secs, luint16 msecs )
{
  m_time =
    ( hours * 60 * 60 * 1000 ) +
    ( mins * 60 * 1000 ) + 
    ( secs * 1000 ) +
    msecs;
}

lint32 LTime::compare ( LTime const & p1 ) const
{
  if ( m_time == p1.m_time )
    return 0;
  else if ( m_time > p1.m_time )
    return 1;
  else
    return 0;
}

bool LTime::equal ( LTime const & p1 ) const
{
  return m_time == p1.m_time;
}

bool LTime::operator ==( const LTime & p1 ) const
{ 
  return equal ( p1 );
}

bool LTime::operator !=( const LTime & p1 ) const
{
  return !equal ( p1 );
}

bool LTime::operator < ( const LTime & p1 ) const
{
  return compare( p1 ) < 0;
}

bool LTime::operator > ( const LTime & p1 ) const
{
  return compare( p1 ) > 0;
}

LTime LTime::operator + ( const LTime & p1 ) const
{
  LTime t_time ( *this );
  t_time.later ( p1 );
  return t_time;
}

LTime LTime::operator - ( const LTime & p1 ) const
{
  LTime t_time ( *this );
  t_time.earlier ( p1 );
  return t_time;
}

LTime & LTime::operator = ( const LTime & p1 )
{
  set ( p1 );
  return *this;
}


luint32 LTime::asMSecs () const
{
  return m_time;
}

luint32 LTime::asSecs () const
{
  return m_time / 1000;
}

luint32 LTime::asMins () const
{
  return m_time / ( 1000 * 60 );
}

luint32 LTime::asHours () const
{
  return m_time / ( 1000 * 60 * 60 );
}

luint32 LTime::getMSecs () const
{
  return asMSecs() % 1000;
}

luint8 LTime::getSecs () const
{
  return asSecs() % 60;
}

luint8 LTime::getMins () const
{
  return asMins () % 60;
}

luint8 LTime::getHours () const
{  
  return asHours() % 24;
}

LTime LTime::diff ( const LTime & p1 ) const
{
  return LTime ( m_time - p1.m_time );
}

void LTime::later ( const LTime & p1 )
{
  m_time += p1.m_time;
}

void LTime::earlier ( const LTime & p1 )
{
  m_time -= p1.m_time;
}

void LTime::setMSecs ( luint32 p1 )
{
  set ( getHours(), getMins(), getSecs(), p1 );
}

void LTime::setSecs ( luint8 p1 )
{
  set ( getHours(), getMins(), p1, getMSecs() );
}

void LTime::setMins ( luint8 p1 )
{
  set ( getHours(), p1, getSecs(), getMSecs() );
}

void LTime::setHours ( luint8 p1 )
{
  set ( p1, getMins(), getSecs(), getMSecs() );
}

void LTime::setAsMSecs ( luint32 p1 )
{
  m_time = p1;
}

void LTime::setAsSecs ( luint32 p1 )
{
  m_time = p1 * 1000;
}

void LTime::setAsMins ( luint32 p1 )
{
  m_time = p1 * 1000 * 60;
}

void LTime::setAsHours ( luint32 p1 )
{
  m_time = p1 * 1000 * 60 * 60;
}


#ifdef HAVE_GETTIMEOFDAY
LTime LTime::now()
{
  struct timeval tv;
  ::gettimeofday ( &tv, NULL );

  LTime t_time;
  t_time.set ( 0, 0, tv.tv_sec%(60*60*24), tv.tv_usec/1000 );
  return t_time;
}
#endif

