/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

/* LCL - The Linux Class Library
 * Copyright (C) 1998-2003 Jens Reimann
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */



#include <lcl.h>

#include <lcl/ltrace.h>
#include <lcl/lstreamproxy.h>
#include <lcl/ldatetime.h>
#include <lcl/ltracechannel.h>
#include <lcl/lregex.h>

#include <string.h>
#include <time.h>

#include <lcl/ltrace_output.h>
#include <lcl/lapp.h>
#include <lcl/llogsystem.h>


// LTraceOutput_OStream

LGuardMutex LTraceOutput_OStream::syncMutex;

LTraceOutput_OStream::LTraceOutput_OStream () :
  outputStream ( lerr ),
  compatibleOutput ( false ),
  syncOutput ( false ),
  localOutput ( NULL ),
  localOutputFD ( NULL )
{
}

LTraceOutput_OStream::~LTraceOutput_OStream ()
{
  closeOutput ();
}

void LTraceOutput_OStream::closeOutput ( void )
{
  LAutoLock _lock ( outputControlLock );

  if ( localOutput )
    {
      delete localOutput;
      localOutput = NULL;
    }
  if ( localOutputFD )
    {
      delete localOutputFD;
      localOutputFD = NULL;
    }
}

void LTraceOutput_OStream::setOutput ( LString const & location )
{
  LAutoLock _lock ( outputControlLock );
  /*
  localOutputFD = new LFile ( location, LFDManager::write | LFDManager::create );

  localOutput = new LFOStream ( *localOutputFD );  
  */
}

void LTraceOutput_OStream::setOptions ( LHash<LString,LString> const & vars )
{
  if ( vars.isSet ( "compatible" ) )
    {
      compatibleOutput = (bool)(long)vars.get ( "compatible", "0" );
    }
  if ( vars.isSet ( "sync" ) )
    {
      syncOutput = (bool)(long)vars.get ( "sync", "0" );
    }
  if ( vars.isSet ( "output" ) )
    {
      setOutput ( vars.get ( "output" ) );
    }
}

void LTraceOutput_OStream::trace ( LTraceMessage const & msg )
{
  //  char * buffer = (char*)malloc ( strlen ( msg.getChannel() ) + strlen(msg.getMessage()) + strlen(msg.getFile() ) + 255 );

  LString buffer;

  if ( compatibleOutput )
    {
      buffer = LString::fromPrintf ( 250, "[%s] %ld.%ld ",(char const*)LDateTime(msg.getTime()).asString("%H:%M:%S %Y-%m-%d"), msg.getPID(), msg.getTID() ) + msg.getChannel() + " -> " + msg.getMessage();
    }
  else
    {
      buffer = LString::fromPrintf ( 4096, "[%s.%ld] [%ld.%ld] [%s:%ld] ", (char const*)LDateTime(msg.getTime()).asString("%Y-%m-%d %H:%M:%S"), msg.getUTime(), msg.getPID(), msg.getTID(), msg.getFile(), msg.getLine() ) + msg.getChannel() + " -> " + msg.getMessage();
    }

  LAutoLock _lock ( outputControlLock );
  if ( syncOutput )
    {
      LAutoLock _lock ( syncMutex );
      outputStream << buffer << endl << flush;
    }
  else
    {
      outputStream << buffer << endl << flush;
    }

  //  free ( buffer );
}
