#include <lcl.h>
#include <lcl/ltracechannel.h>

#include <lcl/lfstream.h>
#include <lcl/lhashiter.h>
#include <lcl/ldatetime.h>
#include <lcl/lfile.h>
#include <lcl/lstreamproxy.h>

#include <stdio.h>

#ifndef PURE_WIN32
#include <lcl/ltracechannel.dgram.h>
#endif

class LTraceChannelIter : public LHashIterator <LString, LManagedPointer<LTraceChannel> >
{
protected:
  const char * module;
  const char * message;

public:
  LTraceChannelIter ( const char * _module, const char * _message ) :
    LHashIterator<LString,LManagedPointer<LTraceChannel> > (),
    module ( _module ),
    message ( _message )
  {
  }

  void applyTo ( LString & key, LManagedPointer<LTraceChannel> & channel )
  {
    if ( channel->isActive () )
      channel->trace ( module, message );
  }
};

LTraceChannelManager * LTraceChannelManager::theInstance = NULL;

LTraceChannelManager::LTraceChannelManager ()
{
}
  
LTraceChannelManager * LTraceChannelManager::getInstance ()
{
  if ( !theInstance )
    theInstance = new LTraceChannelManager ();

  return theInstance;
}

void LTraceChannelManager::trace ( const char * module, const char * message )
{
  LTraceChannelIter ltci ( module, message );
  channelHash.allElementsDo ( ltci );
}

bool LTraceChannelManager::addChannel ( LString const & channelName, LString const & _channel )
{
  LString channel = _channel;
  channel.setDelims ( ":" );
  LString type = channel.word(0);

  LTraceChannel * tChannel = NULL;

  LHash<LString,LString> varHash;

  channel = channel.word(1);
  channel.setDelims ( "&" );

  for ( int i = 0; i<channel.numWords(); i++ ) {
    LString pair = channel.word(i);
    pair.setDelims ( "=" );
    varHash.set ( pair.word(0), pair.word(1) );
  }

  if ( type == "ostream" ) {
    tChannel = new LTraceChannel_OStream ( varHash );
  }
#ifndef PURE_WIN32
  if ( type == "dgram" ) {
    tChannel = new LTraceChannel_DGRAM ( varHash );
  }
#endif

  if ( tChannel ) {
    channelHash.set ( channelName, tChannel );
    return true;
  }
  return false;
}

bool LTraceChannelManager::removeChannel ( LString const & channelName )
{
  if ( channelHash.isSet ( channelName ) )
    channelHash.unset ( channelName );

  return true;
}

LTraceChannel::LTraceChannel () :
  active ( true )
{
}

LTraceChannel::~LTraceChannel ()
{
}

LManagedPointer<LTraceChannel> LTraceChannelManager::getDefaultTraceChannel () const
{
  return channelHash.get ( "default", LManagedPointer<LTraceChannel>() );
}

void LTraceChannel::enable ( bool _active )
{
  active = _active;
}

void LTraceChannel::disable ()
{
  enable ( false );
}

bool LTraceChannel::isActive () const
{
  return active;
}

LTraceChannel_OStream::LTraceChannel_OStream ( LHash<LString,LString> & channelDesc ) :
  ostream ( NULL ),
  valid ( false )
{
  bool isAppend = ( channelDesc.get ( "append", "" ) == "yes" );
  LFileDesc fd;
  if ( channelDesc.isSet ( "fd" ) ) {
    fd = (LFileDescriptor)channelDesc.get("fd");
    fd.setAutoClose ( false );
  } else {
    fd = LFile ( channelDesc.get ( "name" ), LFDManager::write | (isAppend ? LFDManager::append : 0 ) );
    if ( !fd.isValid() )
      lout << "Error opening " << channelDesc.get("name") << endl << flush;
  }
  if ( fd.isValid() ) {
    valid = true;
    ostream = new LFOStream ( fd );
  } else {
    lout << "Couldn't create a valid output stream!" << endl << flush;
  }
}

void LTraceChannel_OStream::trace ( const char * module, const char * message )
{
  if ( !valid )
    return;

  char * buffer1 = (char*)malloc ( 255 + strlen(module) + strlen(message) );
  LDateTime cDate = LDateTime::now();

  sprintf ( buffer1, "[%02d:%02d:%02d %04d-%02d-%02d] %d.%d %s -> %s\n", cDate.getHours(), cDate.getMinutes(), cDate.getSeconds(), cDate.getYear(), cDate.getMonth(), cDate.getDay(), LGetProcessID(), LGetCurrentThreadID(), module, message );

  *ostream << buffer1;
  *ostream << flush;

  free ( buffer1 );

}
