
#include <lcl/ltracefunc.h>

#include <string.h>

// LTraceFunc

LCL_EXPORT LGuardMutex LTraceFunc::stackLock;
LCL_EXPORT LHash<LThreadID, LHeap<LString> > LTraceFunc::stack;

LTraceFunc::LTraceFunc ( const char * _name, const char * _fileName, unsigned long _lineNumber )
{

  name = strdup ( _name );
  fileName = strdup ( _fileName );
  lineNumber = _lineNumber;
  
  LString level = "[" + LString(getLevel()) + "] " + LString::filled(getLevel(),' ');
    
  LTRACEX_MSG ( LTraceMessage ( "func", level + "+" + LString(name), fileName, lineNumber, ::time(0), 0, LGetProcessID(), LGetCurrentThreadID() ) );

  push ( name + LString(" @ ") + LString(fileName) + ":" + LString(lineNumber) );
}

LTraceFunc::~LTraceFunc ()
{
  pop ();

  LString level = "[" + LString(getLevel()) + "] " + LString::filled(getLevel(),' ');
  LTRACEX_MSG ( LTraceMessage ( "func", level + "-" + LString(name), fileName, lineNumber, ::time(0), 0, LGetProcessID(), LGetCurrentThreadID() ) );

  if ( name )
    free ( name );
  if ( fileName )
    free ( fileName );
}

void LTraceFunc::push ( LString const & str )
{
  stackLock.lock ();
  if ( !stack.isSet ( LGetCurrentThreadID() ) )
    {
      stack.set ( LGetCurrentThreadID(), LHeap<LString>() );
    }
  stack.get ( LGetCurrentThreadID() ).add ( str );
  stackLock.unlock ();
}

void LTraceFunc::pop ()
{
  stackLock.lock ();

  if ( stack.isSet ( LGetCurrentThreadID() ) )
    {
      LHeap<LString> & heap = stack.get ( LGetCurrentThreadID() );
      LCursor * cursor = heap.createCursor();
      cursor->setToLast ();
      if ( cursor->isValid() )
	heap.removeElementAt ( cursor );
      delete cursor;
    }
  stackLock.unlock ();
}

unsigned long LTraceFunc::getLevel()
{
  luint32 rc = 0;
  stackLock.lock ();  
  if ( stack.isSet ( LGetCurrentThreadID() ) )
    rc = stack.get ( LGetCurrentThreadID() ).entries();
  stackLock.unlock ();

  return rc;
}


LHeap<LString> LTraceFunc::getStack ( LThreadID tid )
{
  LAutoLock _lock ( stackLock );

  return stack.get ( tid, LHeap<LString>() );
}

void LTraceFunc::dumpStack ( LOStream & out, LThreadID tid )
{
  LAutoLock _lock ( stackLock );

  LHeap<LString> theStack = getStack ( tid );
  out << "------------------ LCL Stack Dump -------------------" << endl;
  luint32 level = 0;
  LCursor * cursor = theStack.createCursor();
  forcursor ( cursor )
    {
      out << "[" << level << "] " << theStack.elementAt ( cursor ) << endl;
      level++;
    }
  out << "------------------  End of Dump   -------------------" << endl << flush;
}
