/* -*-mode: c++; coding: iso-8859-2-unix; -*- */

#include <lcl/lvariant.h>

#include <lcl/lstreamproxy.h>

#define DT2NAME_TYPE(x) case L_VAR_VALUE::x: return #x;

LString dt2name ( L_VAR_VALUE::DataType type )
{
  switch ( type )
    {
      DT2NAME_TYPE(DT_EMPTY)
	DT2NAME_TYPE(DT_BOOL)
	
	DT2NAME_TYPE(DT_INT8)
	DT2NAME_TYPE(DT_INT16)
	DT2NAME_TYPE(DT_INT32)
	DT2NAME_TYPE(DT_UINT8)
	DT2NAME_TYPE(DT_UINT16)
	DT2NAME_TYPE(DT_UINT32)
	
	DT2NAME_TYPE(DT_STRING)
	
	DT2NAME_TYPE(DT_DOUBLE)

    default:
      return "<UNKOWN>";
    }
}

LTypeMismatchException::LTypeMismatchException ( L_VAR_VALUE::DataType _srcType, L_VAR_VALUE::DataType _dstType, const char * _filename, unsigned long _line ) :
  LBasicException ( _filename, _line ),
  srcType ( _srcType ),
  dstType ( _dstType )
{
}

void LTypeMismatchException::log () const
{
  LBasicException::log ();
  lerr << " >> Type Mismatch Exception!" << endl;;
  lerr << "  >> Source Type: " << dt2name(srcType) << endl;
  lerr << "  >> Destination Type: " << dt2name(dstType) << endl;
  lerr << flush;
}


LVariant::LVariant ()
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_EMPTY;
}

LVariant::LVariant ( LVariant const & p1 ) :
  m_value ( p1.m_value )
{
}

LVariant::LVariant ( LString const & str )
{
  L_VAR_VALUE_TYPE(&m_value ) = L_VAR_VALUE::DT_STRING;
  L_VAR_VALUE_DT_STRING(&m_value) = str;
}

LVariant::LVariant ( bool p1 )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_BOOL;
  L_VAR_VALUE_DT_BOOL(&m_value) = p1;
}

LVariant::LVariant ( double p1 )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_DOUBLE;
  L_VAR_VALUE_DT_DOUBLE(&m_value) = p1;
}

LVariant::LVariant ( lint8 p1 )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_INT8;
  L_VAR_VALUE_DT_INT8(&m_value) = p1;
}

LVariant::LVariant ( lint16 p1 )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_INT16;
  L_VAR_VALUE_DT_INT16(&m_value) = p1;
}

LVariant::LVariant ( lint32 p1 )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_INT32;
  L_VAR_VALUE_DT_INT32(&m_value) = p1;
}

LVariant::LVariant ( luint8 p1 )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_UINT8;
  L_VAR_VALUE_DT_UINT8(&m_value) = p1;
}

LVariant::LVariant ( luint16 p1 )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_UINT16;
  L_VAR_VALUE_DT_UINT16(&m_value) = p1;
}

LVariant::LVariant ( luint32 p1 )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_UINT32;
  L_VAR_VALUE_DT_UINT32(&m_value) = p1;
}

LVariant::~LVariant ()
{
}

void LVariant::clear ()
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_EMPTY;
}

void LVariant::copyFrom ( LVariant const & p1 )
{
  m_value = p1.m_value;
}

L_VAR_VALUE::DataType LVariant::getType () const
{
  return L_VAR_VALUE_TYPE(&m_value);
}

LVariant & LVariant::operator = ( LVariant const & p1 )
{
  copyFrom ( p1 );
  return *this;
}

LVariant & LVariant::operator = ( bool value )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_BOOL;
  //  L_VAR_VALUE_DT_BOOL(&m_value) = value;
  L_VAR_DATA_DT_BOOL(&m_value.data) = value;
  return *this;
}

LVariant & LVariant::operator = ( char const * value )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_STRING;
  L_VAR_VALUE_DT_STRING(&m_value) = value;
  return *this;
}

LVariant & LVariant::operator = ( LString const & value )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_STRING;
  L_VAR_VALUE_DT_STRING(&m_value) = value;
  return *this;
}

LVariant & LVariant::operator = ( lint8 value )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_INT8;
  L_VAR_VALUE_DT_INT8(&m_value) = value;
  return *this;
}

LVariant & LVariant::operator = ( lint16 value )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_INT16;
  L_VAR_VALUE_DT_INT16(&m_value) = value;
  return *this;
}

LVariant & LVariant::operator = ( lint32 value )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_INT32;
  L_VAR_VALUE_DT_INT32(&m_value) = value;
  return *this;
}

LVariant & LVariant::operator = ( luint8 value )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_UINT8;
  L_VAR_VALUE_DT_UINT8(&m_value) = value;
  return *this;
}

LVariant & LVariant::operator = ( luint16 value )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_UINT16;
  L_VAR_VALUE_DT_UINT16(&m_value) = value;
  return *this;
}

LVariant & LVariant::operator = ( luint32 value )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_UINT32;
  L_VAR_VALUE_DT_UINT32(&m_value) = value;
  return *this;
}

LVariant & LVariant::operator = ( double value )
{
  L_VAR_VALUE_TYPE(&m_value) = L_VAR_VALUE::DT_DOUBLE;
  L_VAR_VALUE_DT_DOUBLE(&m_value) = value;
  return *this;
}

// Bool

bool LVariant::asBool () const
{

  LString tmpStr;

  switch ( L_VAR_VALUE_TYPE(&m_value) )
    {

    case L_VAR_VALUE::DT_EMPTY:
      return false;

    case L_VAR_VALUE::DT_BOOL:
      return L_VAR_VALUE_DT_BOOL(&m_value);

    case L_VAR_VALUE::DT_INT8:
      return (bool)L_VAR_VALUE_DT_INT8(&m_value);
    case L_VAR_VALUE::DT_INT16:
      return (bool)L_VAR_VALUE_DT_INT16(&m_value);
    case L_VAR_VALUE::DT_INT32:
      return (bool)L_VAR_VALUE_DT_INT32(&m_value);
    case L_VAR_VALUE::DT_UINT8:
      return (bool)L_VAR_VALUE_DT_UINT8(&m_value);
    case L_VAR_VALUE::DT_UINT16:
      return (bool)L_VAR_VALUE_DT_UINT16(&m_value);
    case L_VAR_VALUE::DT_UINT32:
      return (bool)L_VAR_VALUE_DT_UINT32(&m_value);
    case L_VAR_VALUE::DT_DOUBLE:
      return (bool)L_VAR_VALUE_DT_DOUBLE(&m_value);

    case L_VAR_VALUE::DT_STRING:
      tmpStr = L_VAR_VALUE_DT_STRING(&m_value);

      if ( tmpStr.length() == 0 )
	{
	  return false;
	}

      if ( tmpStr.isDigits() || tmpStr.isHexDigits() )
	{
	  return (bool)(long)tmpStr;
	}

      tmpStr = tmpStr.toUpper();

      if ( tmpStr == "TRUE" )
	{
	  return true;
	}
      return false;

    default:
      throw LTypeMismatchException ( L_VAR_VALUE_TYPE(&m_value), L_VAR_VALUE::DT_BOOL, __FILE__, __LINE__ );
      MSTHROW(bool);
      break;
    }
}

// String

LString LVariant::asString () const
{

  switch ( L_VAR_VALUE_TYPE(&m_value) )
    {

    case L_VAR_VALUE::DT_BOOL:
      return LString((long)L_VAR_VALUE_DT_BOOL(&m_value));
      
    case L_VAR_VALUE::DT_INT8:
      return LString((long)L_VAR_VALUE_DT_INT8(&m_value));
    case L_VAR_VALUE::DT_INT16:
      return LString((long)L_VAR_VALUE_DT_INT16(&m_value));
    case L_VAR_VALUE::DT_INT32:
      return LString((long)L_VAR_VALUE_DT_INT32(&m_value));

    case L_VAR_VALUE::DT_UINT8:
      return LString((unsigned long)L_VAR_VALUE_DT_UINT8(&m_value));
    case L_VAR_VALUE::DT_UINT16:
      return LString((unsigned long)L_VAR_VALUE_DT_UINT16(&m_value));
    case L_VAR_VALUE::DT_UINT32:
      return LString((unsigned long)L_VAR_VALUE_DT_UINT32(&m_value));

    case L_VAR_VALUE::DT_DOUBLE:
      return LString((double)L_VAR_VALUE_DT_DOUBLE(&m_value));

    case L_VAR_VALUE::DT_STRING:
      return L_VAR_VALUE_DT_STRING(&m_value);

    case L_VAR_VALUE::DT_EMPTY:
      return LString();

    default:
      throw LTypeMismatchException ( L_VAR_VALUE_TYPE(&m_value), L_VAR_VALUE::DT_STRING, __FILE__, __LINE__ );
      MSTHROW(LString);
      break;
    }
}

luint8 LVariant::asUInt8 () const
{

  switch ( L_VAR_VALUE_TYPE(&m_value) )
    {

    case L_VAR_VALUE::DT_BOOL:
      return (luint8)L_VAR_VALUE_DT_BOOL(&m_value);
      
    case L_VAR_VALUE::DT_INT8:
      return (luint8)L_VAR_VALUE_DT_INT8(&m_value);
    case L_VAR_VALUE::DT_INT16:
      return (luint8)L_VAR_VALUE_DT_INT16(&m_value);
    case L_VAR_VALUE::DT_INT32:
      return (luint8)L_VAR_VALUE_DT_INT32(&m_value);

    case L_VAR_VALUE::DT_UINT8:
      return (luint8)L_VAR_VALUE_DT_UINT8(&m_value);
    case L_VAR_VALUE::DT_UINT16:
      return (luint8)L_VAR_VALUE_DT_UINT16(&m_value);
    case L_VAR_VALUE::DT_UINT32:
      return (luint8)L_VAR_VALUE_DT_UINT32(&m_value);

    case L_VAR_VALUE::DT_DOUBLE:
      return (luint8)L_VAR_VALUE_DT_DOUBLE(&m_value);

    case L_VAR_VALUE::DT_STRING:
      return (unsigned short)L_VAR_VALUE_DT_STRING(&m_value);

    case L_VAR_VALUE::DT_EMPTY:
      return 0;

    default:
      throw LTypeMismatchException ( L_VAR_VALUE_TYPE(&m_value), L_VAR_VALUE::DT_STRING, __FILE__, __LINE__ );
      MSTHROW(LString);
      break;
    }
}

luint16 LVariant::asUInt16 () const
{
  L_VAR_VALUE_CONVERT(luint16,&m_value,0,L_VAR_VALUE::DT_UINT16);
  MSTHROW(luint16);
}

luint32 LVariant::asUInt32 () const
{
  L_VAR_VALUE_CONVERT(luint32,&m_value,0,L_VAR_VALUE::DT_UINT32);
  MSTHROW(luint32);
}

lint8 LVariant::asInt8 () const
{
  L_VAR_VALUE_CONVERT(lint8,&m_value,0,L_VAR_VALUE::DT_INT8);
  MSTHROW(lint8);
}

lint16 LVariant::asInt16 () const
{
  L_VAR_VALUE_CONVERT(lint16,&m_value,0,L_VAR_VALUE::DT_INT16);
  MSTHROW(lint16);
}

lint32 LVariant::asInt32 () const
{
  L_VAR_VALUE_CONVERT(lint32,&m_value,0,L_VAR_VALUE::DT_INT32);
  MSTHROW(lint32);
}

double LVariant::asDouble () const
{
  L_VAR_VALUE_CONVERT(double,&m_value,0,L_VAR_VALUE::DT_DOUBLE);
  MSTHROW(double);
}


LVariant::operator bool () const
{
  return asBool ();
}

LVariant::operator LString () const
{
  return asString ();
}

LVariant::operator LString ()
{
  return asString ();
}

LVariant::operator double () const
{
  return asDouble ();
}

LVariant::operator lint8 () const
{
  return asInt8 ();
}
LVariant::operator lint16 () const
{
  return asInt16 ();
}
LVariant::operator lint32 () const
{
  return asInt32 ();
}

LVariant::operator luint8 () const
{
  return asUInt8 ();
}
LVariant::operator luint16 () const
{
  return asUInt16 ();
}
LVariant::operator luint32 () const
{
  return asUInt32 ();
}

bool LVariant::isBool () const
{
  return L_VAR_VALUE_TYPE(&m_value) == L_VAR_VALUE::DT_BOOL;
}

bool LVariant::isString () const
{
  return L_VAR_VALUE_TYPE(&m_value) == L_VAR_VALUE::DT_STRING;
}

bool LVariant::isDouble () const
{
  return L_VAR_VALUE_TYPE(&m_value) == L_VAR_VALUE::DT_DOUBLE;
}

bool LVariant::operator == ( LVariant const & p1 ) const
{
  switch ( L_VAR_VALUE_TYPE(&m_value) )
    {
    case L_VAR_VALUE::DT_STRING:
      switch ( L_VAR_VALUE_TYPE(&p1.m_value) )
	{
	case L_VAR_VALUE::DT_STRING:
	  return L_VAR_VALUE_DT_STRING(&m_value) == L_VAR_VALUE_DT_STRING(&p1.m_value);
	case L_VAR_VALUE::DT_INT8:
	case L_VAR_VALUE::DT_INT16:
	case L_VAR_VALUE::DT_INT32:
	  return p1.asInt32() == asInt32();
	case L_VAR_VALUE::DT_UINT8:
	case L_VAR_VALUE::DT_UINT16:
	case L_VAR_VALUE::DT_UINT32:
	  return p1.asUInt32() == asUInt32();
	case L_VAR_VALUE::DT_DOUBLE:
	  return p1.asDouble() == asDouble();
	case L_VAR_VALUE::DT_BOOL:
	  return p1.asBool() == asBool();
	default:
	  throw LTypeMismatchException ( L_VAR_VALUE_TYPE(&m_value), L_VAR_VALUE_TYPE(&p1.m_value) , __FILE__, __LINE__ );
	  MSTHROW(bool);
	  break;
	}

    case L_VAR_VALUE::DT_BOOL:
      return p1.asBool() == L_VAR_VALUE_DT_BOOL(&m_value);

    case L_VAR_VALUE::DT_INT8:
      return p1.asInt8() == L_VAR_VALUE_DT_INT8(&m_value);
    case L_VAR_VALUE::DT_INT16:
      return p1.asInt16() == L_VAR_VALUE_DT_INT16(&m_value);
    case L_VAR_VALUE::DT_INT32:
      return p1.asInt32() == L_VAR_VALUE_DT_INT32(&m_value);

    case L_VAR_VALUE::DT_UINT8:
      return p1.asUInt8() == L_VAR_VALUE_DT_UINT8(&m_value);
    case L_VAR_VALUE::DT_UINT16:
      return p1.asUInt16() == L_VAR_VALUE_DT_UINT16(&m_value);
    case L_VAR_VALUE::DT_UINT32:
      return p1.asUInt32() == L_VAR_VALUE_DT_UINT32(&m_value);

    case L_VAR_VALUE::DT_DOUBLE:
      return p1.asBool() == L_VAR_VALUE_DT_BOOL(&m_value);

    default:
      throw LTypeMismatchException ( L_VAR_VALUE_TYPE(&m_value), L_VAR_VALUE_TYPE(&p1.m_value) , __FILE__, __LINE__ );
      MSTHROW(bool);
      break;
    }
}

LCL_EXPORT NAMESPACE_STD(ostream) & operator << ( NAMESPACE_STD(ostream) & os, LVariant const & p1 )
{
  os << p1.asString();
  return os;
}

LCL_EXPORT LOStream & operator << ( LOStream & los, LVariant const & p1 )
{
  los << p1.asString();
  return los;
}
