/* *************************************************************************************************
   |      File: lxcpt.cc
   |      Contains: base class LException
   |      Created: 07.06.1998
   |      Last Changed: 07.06.1998
   |      Version: 0.0
   |         
   |      ToDo: + just anything
   |            + exceptions !! (coming!)
   |
   ************************************************************************************************* */

/* ************************************************************************************************ *
 * Change log:                                                                                      *
 * -------------                                                                                    *
 *                                                                                                  *
 *   -> [v0.00] - 07.06.1998 - Created file and started coding                                      *
 * ************************************************************************************************ */


// **** LCL Includes

#include <lcl.h>

#include <lcl/lxcpt.h>
#include <lcl/lstreamproxy.h>
#include <lcl/lpointer.h>
#include <lcl/ltrace.h>

/*
LInfoException & LInfoException::set ( LString const & key, LString const & value )
{
  info.set ( key, value );
  return *this;
}
  
void LInfoException::log ( ) const
{
  LSet<LString> keys = info.getKeys();
  LManagedPointer<LCursor> cursor = keys.createCursor ();
  forcursor ( cursor )
    {
      lerr << "  " << keys.elementAt ( cursor ) << " -> " << info.get(keys.elementAt(cursor), "" ) << endl;
    }
  lerr << flush;
}
*/

LFileInfoException::LFileInfoException ( char const * _filename, lint32 _line ) :
  filename ( strdup ( _filename ) ),
  line ( _line )
{
}

LFileInfoException::LFileInfoException ( LFileInfoException const & p1 ) :
  filename ( p1.filename ? strdup ( p1.filename ) : NULL ),
  line ( p1.line )
{
}

LFileInfoException::~LFileInfoException ()
{
  if ( filename )
    free ( filename );
}

void LFileInfoException::log ( ) const
{
  lerr << "Exception at " << (filename ? filename : "<unkown>" ) << ":" << line << endl << flush;    
}

// LStackInfoException 

LStackInfoException::LStackInfoException () :
  stack_v ( NULL )
{
  LHeap<LString> heap = LTraceFunc::getStack ();
  stack_v = (char**)::malloc ( sizeof(char*) * (heap.entries()+1) );
  LCursor * cursor = heap.createCursor ();
  luint32 i = 0;
  forcursor ( cursor )
    {
      stack_v[i] = strdup ( heap.elementAt ( cursor ) );
      i++;
    }
  stack_v[i] = NULL;
  delete cursor;
}

LStackInfoException::LStackInfoException ( LStackInfoException const & p1 ) :
  stack_v ( NULL )
{
  if ( p1.stack_v )
    {
      luint32 i = 0;
      while ( p1.stack_v[i] )
	{
	  i++;
	}
      stack_v = (char**)::malloc ( sizeof(char*) * (i+1) );
      i = 0;
      while ( p1.stack_v[i] )
	{
	  stack_v[i] = ::strdup ( p1.stack_v[i] );
	}
      stack_v[i] = NULL;
    }
}

LStackInfoException::~LStackInfoException ()
{
  if ( stack_v )
    {
      luint32 i = 0;
      while ( stack_v[i] )
	{
	  free ( stack_v[i] );
	  i++;
	}
      free ( stack_v );
    }
}

void LStackInfoException::log ( ) const
{
  lerr << "LCL Stack Dump: " << endl;

  if ( stack_v )
    {
      luint32 i = 0;
      while ( stack_v[i] )
	{
	  lerr << i << " - " << stack_v[i] << endl;
	  i++;
	}
    }
  lerr << "End Of Stack Dump!" << endl;
  lerr << flush;
}

// LBasicException

LBasicException::LBasicException ( const char * _filename, unsigned long _line, const char * _xreason ) :
  LFileInfoException ( _filename, _line ),
  xreason ( strdup ( _xreason ) )
{
}

LBasicException::LBasicException ( LBasicException const & p1 ) :
  LFileInfoException ( p1 ),
  xreason ( p1.xreason ? strdup ( p1.xreason ) : NULL )
{
}

LBasicException::~LBasicException ()
{
  if ( xreason )
    free ( xreason );
}

void LBasicException::log () const
{
  lerr << "LCL Exception [" << (unsigned long)LGetCurrentThreadID() << "]" << endl;

  LFileInfoException::log ();
  LStackInfoException::log ();

  lerr << " -> Reason: " << xreason << endl;

  //  LInfoException::log ();

  lerr << flush;
}


LOutOfMemoryException::LOutOfMemoryException ( const char * _filename, unsigned long _line ) :
  LBasicException ( _filename, _line, "Out of memory" )
{
}

void LOutOfMemoryException::log () const
{
  LBasicException::log ();
  lerr << " >> Out of memory exception!" << endl << flush;
}

LIOException::LIOException ( const char * _filename, unsigned long _line ) :
  LBasicException ( _filename, _line, "IO Exception" )
{
}

void LIOException::log () const
{
  LBasicException::log ();
  lerr << " >> IO Exception!" << endl << flush;
}


LNullPointerException::LNullPointerException ( const char * _filename, unsigned long _line ) :
  LBasicException ( _filename, _line, "NULL Pointer" )
{
}

void LNullPointerException::log () const
{
  LBasicException::log ();
  lerr << " >> Null-Pointer Exception!" << endl << flush;
}

LWrongParameterException::LWrongParameterException ( const char * _filename, unsigned long _line, char const * _reason ) :
  LBasicException ( _filename, _line, "Wrong parameter" ),
  reason ( strdup ( _reason ) )
{
}

LWrongParameterException::~LWrongParameterException ()
{
  free ( reason );
}

void LWrongParameterException::log ( ) const
{
  LBasicException::log ();
  lerr << " >> Wrong Parameter Exception!" << endl;;
  lerr << "  >> " << reason << endl;
  lerr << flush;
}

// LAssertionException

LAssertionException::LAssertionException ( char const * _filename, unsigned long _line, char const * _assertion ) :
  LBasicException ( _filename, _line, "Assertion failed" ),
  assertion ( _assertion ? strdup ( _assertion ) : NULL )
{
}

LAssertionException::~LAssertionException ()
{
  if ( assertion )
    free (assertion);
}

void LAssertionException::log ( ) const
{
  LBasicException::log ();
  lerr << " -> Assertion: " << (char const*)(assertion ? assertion : "<unkown>" ) << endl;
  lerr << flush;
}
