#include <lcl_common.h>

void dumpIntSet ( LSet<int> intSet )
{
  LManagedPointer<LCursor> cursor1 = intSet.createCursor ();
  for ( cursor->setToFirst(); cursor->isValid(); cursor->setToNext() )
    {
      lout << intSet.elementAt ( cursor );
    }
}

template < class Type >
void dumpAnySet ( LSet<Type> anySet )
{
  LManagedPointer<LCursor> cursor1 = anySet.createCursor ();
  for ( cursor->setToFirst(); cursor->isValid(); cursor->setToNext() )
    {
      lout << anySet.elementAt ( cursor );
    }
}

template < class Type >
void dumpAnyHeap ( LHeap<Type> anyHeap )
{
  LManagedPointer<LCursor> cursor1 = anyHeap.createCursor ();
  for ( cursor->setToFirst(); cursor->isValid(); cursor->setToNext() )
    {
      lout << anyHeap.elementAt ( cursor );
    }  
}

int main ( int argc, char * argv [] )
{
  try
    {
      theApp.init ( argc, argv );
      theApp.processLCLParameters ();

      // Do your application stuff here...


      // ===============>>>>>>>>>>> LSet <<<<<<<<<<==============
      // Create to new sets
      LSet<int> intSet;
      LSet<LString> stringSet;

      // Add some data
      intSet.add ( 1 );
      intSet.add ( 2 );
      stringSet.add ( "Hello World" );
      stringSet.add ( "Hello World" ); // This will return "false" since the element is already in there
      stringSet.add ( "Foo Bar" );

      // Dump
      dumpIntSet ( intSet );
      dumpAnySet ( stringSet );

      // Remove one element and redump
      /*
	The element can be removed since LSet<Type> supports equality. Thefore the LSet<Type>
	can search through its list by comparing item_to_delete==current_item.
       */
      intSet.removeElement ( 1 );
      dumpAnySet ( intSet );

      // ===============>>>>>>>>>>> LHeap <<<<<<<<<<==============

      // Now we create two heaps
      /*
	Heaps do not support anything. They only maintain the order how they where inserted.	
       */
      LHeap<int> intHeap;
      LHeap<LString> stringHeap;

      // Now add some data
      intHeap.add ( 1 );
      intHeap.add ( 2 );
      stringHeap.add ( "Hello World" );
      stringHeap.add ( "Hello World" ); // This will be ok!
      stringHeap.add ( "Foo Bar" );

      dumpAnyHeap ( intHeap );
      dumpAnyHeap ( stringHeap );

      lout << "------------";
      LString temp;
      cursor = stringHeap.createCursor ();
      forcursor ( cursor )
	{
	  lout << stringHeap.elementAt ( cursor ) << endl;
	  if ( stringHeap.elementAt ( cursor ) == temp )
	    {
	      stringHeap.removeElementAt ( cursor );
	      // Now the cursor is invalid! Since the collection was altered!
	    }
	  temp = stringHeap.elementAt ( cursor );
	}
      lout << "------------";
      // The printout will show
      /*
	------------
	Hello World
	Hello World
	------------
       */
      // "Foor Bar" will not be shown since the cursor was invalidated before.
      lout << "stringHeap entries: " << stringHeap.entries() << endl;
      stringHeap.clear ();
      lout << "stringHeap entries: " << stringHeap.entries() << endl;
    }

  catch ( LBasicException & e )
    {
      e.log ();
      return 1;
    }
  
  return 0;
}
