#include <lcl_common.h>

void dump_words_easy ( LString const & _str, LString const & delims = " \t\n\r", LString const & qmarks = "" )
{

  LString str = _str;

  str.setDelims ( delims );
  str.setQMarks ( qmarks );

  unsigned int i;
  for ( i = 0; i<str.numWords(); i++ )
    {
      lout << "Word #" << i << ": " << str.word(i) << endl;
    }
}

void dump_words_fast ( LString const & str, LString const & delims = " \t\n\r", LString const & qmarks = "" )
{
  LStringTokenizer tok ( delims, qmarks );
  tok.tokenize ( str );

  unsigned int i;
  for ( i = 0; i<tok.numWords(); i++ )
    {
      lout << "Word #" << i << ": " << tok.word(i) << endl;
    }
}

int main ( int argc, char * argv[] )
{
  try
    {
      theApp.init ( argc, argv );
      theApp.processLCLParameters ();

      // Hello World!
      // ----- ------
      dump_words_easy ( "Hello World!" );
      dump_words_fast ( "Hello World!" );

      //   Hello   World!!  
      //   -----   -------
      dump_words_easy ( "  Hello   World!!  " );
      dump_words_fast ( "  Hello   World!!  " );

      //   ' Hello World!! '
      //    ---------------
      dump_words_fast ( "  ' Hello World!! '  ", " \n\r\t", "'" );
    }

  catch ( LBasicException & e )
    {
      e.log ();
      return 1;
    }

  return 0;
}
