
#include <lapp.h>
#include <lcrypt.xor.h>
#include <lstreamproxy.h>

#include <ltrace.h>

void encode ( LString const & _key, LString const & filename )
{
  LBuffer buffer;
  buffer.loadFile ( filename );

  LTRACE ( "encode", "encoding " + LString(buffer.getSize()) + " bytes!" );

  LCryptSecretKey key;
  key.set ( _key );

  LCryptModule_XOR cmod ( key );

  LTRACE ( "encode", "crypto module set up!" );

  cmod.encode ( buffer );

  LTRACE ( "encode", "saving " + LString(buffer.getSize()) + " bytes!" );

  buffer.saveFile ( filename );
}

int main ( int argc, char * argv [] )
{
  theApp.init ( argc, argv );
  theApp.processLCLParameters ();

  if ( !theApp.hasParam ( "k" ) ) {
    lout << "Key not set! use -k <key>" << endl << flush;
    return 1;
  }

  LString key = theApp.getParam ( "k" );

  LHeap<LString> list = theApp.getFileList ();

  LTRACE ( "crypt", "processing " + LString(list.entries()) + " file(s)!" );

  LCursor * cursor = list.createCursor();

  for ( cursor->setToFirst(); cursor->isValid(); cursor->setToNext() ) {
    encode ( key, list.elementAt ( cursor ) );
  }

  delete cursor;
  return 0;
}

