#include <lcl.h>
#include <lcl/lstring.h>
#include <lcl/lstringreplace.h>
#include <lcl/lapp.h>
#include <iostream.h>

#include "lcl_test.h"

LCLTester lclTester;

#ifndef _LCL_SYS_WIN32_

#include <sys/time.h>
struct timeval perf_tv;

void perf_start ( void )
{
  gettimeofday ( &perf_tv, NULL );
}

void perf_end ( void )
{
  struct timeval perf_tv_end;
  gettimeofday ( &perf_tv_end, NULL );

  double s1 = (perf_tv.tv_sec * 1000) + (perf_tv.tv_usec/1000);
  double s2 = (perf_tv_end.tv_sec * 1000) + (perf_tv_end.tv_usec/1000);

  printf ( "%d\n", (long)(s2-s1) );
}
#else
#define perf_start()
#define perf_end()
#endif


void testWords ( LString string, LString const & del = " \n\r\t", LString const & qm = "\"" )
{
  string.setDelims ( del );
  string.setQMarks ( qm );

  int i;
  cout << "'" << string << "'\n";
  cout << "Words: " << string.numWords() << "\n";

  for ( i=0; i<string.numWords(); i++ ) {
    cout << "'" << string.word(i) << "' ";
  }
  cout << "\n\n\n";
}

void testConv ( LString string )
{
  cout << "String is : '" << string << "'\n";
}

void testParam ( LString const & str)
{
  LString str1 = "Hallo " + str + "Test";
  cout << str1 << endl << flush;
}


void testReplaceVars ( )
{
  cout << " -->> LString::replaceVars" << endl;
  cout << " ========================================" << endl;

  LHash<LString,LString> hash;
  hash.set ( "VAR1", "VALUE1" );
  hash.set ( "VAR2", "VALUE2" );
  hash.set ( "VAR3", "VALUE3" );

  cout << LString("Let's replace! -> $(VAR1) - $(VAR2) - $(VAR3:EMPTY1) - $(VAR4:EMPTY2) ").replaceVars( hash ) << endl;

  cout << " ========================================" << endl;
  cout << flush;
}

void testReplaceWithEnvs ()
{
  cout << " -->> LString::replaceWithEnvs" << endl;
  cout << " ========================================" << endl;
  cout << LString("My $(HOME:/) is my castle! And $(PATH) is huge...").replaceWithEnvs () << endl;
  cout << LString("This is totally $(UNDEFINED:UNDEFINED) ...").replaceWithEnvs () << endl;
  cout << " ========================================" << endl;
  cout << flush;
}

void testReplacer ( LString const & str, LHash<LString,LString> const & hash, LString const & res )
{
  LString test = str.replaceVars(hash);
  cout << "Replacing: '" << str << "' --> '" << test << "'" << " == '" << res << "'" << endl;

  lclTester.addResult ( test == res );
}

void testReplacerX ( LString const & exp, LString const & str, LHash<LString,LString> const & hash, LString const & res )
{
  LString test = str.replace ( LStringHashReplacer ( hash, true ), exp );
  cout << "Replacing('" << exp << "'): '" << str << "' --> '" << test << "'" << " == '" << res << "'" << endl;

  lclTester.addResult ( test == res );
}

void testReplacerExp ( LString const & exp, LString const & head, LString const & tail )
{
  LHash<LString,LString> hash;

  hash.clear ();
  hash.set ( "TEST1", "--TEST1--" );
  hash.set ( "TEST2", "" );

  testReplacerX ( exp, head+"TEST1"+tail, hash, "--TEST1--" );
  testReplacerX ( exp, head+"TEST1"+tail+head+"TEST1"+tail, hash, "--TEST1----TEST1--" );
  testReplacerX ( exp, head+"TEST1"+tail+head+"TEST2"+tail+head+"TEST1"+tail, hash, "--TEST1----TEST1--" );
  hash.set ( "X", "1" );
  testReplacerX ( exp, head+"TEST"+head+"X"+tail+""+tail+" "+head+"TEST"+head+"X"+tail+""+tail, hash, "--TEST1-- --TEST1--" );
  testReplacerX ( exp, head+"TEST"+head+"Y:-1"+tail+""+tail, hash, "--TEST1--" );

  testReplacerX ( exp, head+"TEST1"+tail+head+"TEST2:-   "+tail+head+"TEST1"+tail, hash, "--TEST1--   --TEST1--" );
  testReplacerX ( exp, head+"Y:-"+head+"Y:-"+head+"X"+tail+""+tail+""+tail, hash, "1" );
  testReplacerX ( exp, head+"#Y:-"+head+"Y:-"+head+"X:+1234"+tail+""+tail+""+tail, hash, "4" );
  testReplacerX ( exp, head+"X?1:2"+tail, hash, "1" );
  testReplacerX ( exp, head+"Y?1:2"+tail, hash, "2" );
}

int main ( int argc, char *argv[])
{
  theApp.init ( argc, argv );
  theApp.processLCLParameters ();

  LString str1;
  LString str2 ( "Hallo" );
  LString str3 ( str2 );

  cout << str1 << "\n";
  cout << str2 << "\n";
  cout << str3 << "\n";

  str1 = str3;
  cout << str1 << "\n";
  
  str2 = "Greetings to Phiber Optik\n";
  cout << str2;

  str1 = (char*)NULL;
  if ( str1 == "any String") {
    cout << "MATCH!\n";
  }
  str1 = "Linux";
  str2 = str1;
  if ( str1 == str2 ) {
    cout << "They match\n";
  }

  str1[4] = 'X';
  cout << str1 << "\n";

  str1 = "One";
  str2 = "Two";
  str3 = str1+" "+str2;
  cout << str3 << "\n";

  testWords ( "Have you read \"Masters of Deception\". My favourite book!" );
  testWords ( "  This one \"is a bit\" crazy!! " );
  testWords ( "  I    have    some    whitespaces!" );
  testWords ( "I'm ending with some whitespaces...       ");
  testWords ( "I am quite normal");
  testWords ( "I_have_different_delimiters", "_" );
  testWords ( "I_have_|also_different|_qmarks", "_", "|" );

  str1 = "123456";
  cout << "From 0: " << str1.subString(0) << "\n";
  cout << "From 1: " << str1.subString(1) << "\n";
  cout << "From 4: " << str1.subString(4) << "\n";
  cout << "From 9: " << str1.subString(9) << "\n";

  str1 = " Welt!\n";

  cout << "Hallo"+str1;

  testConv ( 1UL );
  testConv ( 0xFFFFFFFFL );

  str1 = "12345";
  cout << str1 << "\n";
  str1.removeLastChar();
  str1.removeFirstChar();
  cout << str1 << "\n";

  str1 = "0123456789";
  cout << str1 << "\n";
  cout << str1.subString ( 2, 2 ) << "\n";
  cout << str1.subString ( 7, 10 ) << "\n";
  cout << str1.subString ( 10, 1 ) << "\n";

  LHash<LString, LString> hashTable;
  hashTable.set("NAME","Stavros");
  hashTable.set("CITY","New York");

  str1 = "Hallo my name is $(NAME) and I'm living in $(CITY).\nI earn 100$ each weak (50$+50$). $(GREETS) $(GREETS) C ya...";
  cout << "String is:\n" << str1;
  cout << "With replaced vars:\n" << str1.replaceVars ( hashTable );

  cout << "\nLet's test to + operator\n";

  testParam ( " meister lampe " );

  testReplaceVars ();
  testReplaceWithEnvs ();
 
  double dval = 123456789012.0;
  cout << LString::fromDouble ( dval, -1, 0 ) << endl;
  cout << LString::fromDouble ( (double)LString::fromDouble ( dval, 0, 0 ) ) << endl;

  cout << "String append test..." << endl << flush;
  perf_start ();

  str1 = "123";
  int i;
  for ( i = 0; i<1024*10; i++ )
    {
      str1 += ".";
    }
  perf_end ();

  cout << "...done" << endl << flush;

  str1 = "1";
  for ( i = 2; i<129; i++ )
    {
      str1 += LString(i%10);
      cout << str1 << endl << flush;;
    }

  while ( str1.length() )
    {
      str1 = str1.removeLastChar ();
      cout << str1 << endl << flush;;
    }

  LString str_len;
  LSTR_TEST_A ( lclTester, str_len.length(), str_len=str_len, 0 );
  LSTR_TEST_A ( lclTester, str_len.length(), str_len="", 0 );
  LSTR_TEST_A ( lclTester, str_len.length(), str_len="1", 1 );
  LSTR_TEST_A ( lclTester, str_len.length(), str_len="1234567890", 10 );
  LSTR_TEST_A ( lclTester, str_len.length(), str_len=(char const*)NULL, 0 );
  LSTR_TEST_A ( lclTester, str_len.length(), str_len=LString("1234567890"), 10 );
  LSTR_TEST_A ( lclTester, str_len.length(), str_len+="1234567890", 20 );

  LSTR_TEST ( lclTester, LString((char*)NULL), "" );
  LSTR_TEST ( lclTester, LString(""), "" );
  LSTR_TEST ( lclTester, LString("123"), "123" );
  LSTR_TEST ( lclTester, LString(123), "123" );
  LSTR_TEST ( lclTester, LString(LString(123)), "123" );

  lclTester.test ( "LString::fromPrintf ( 1000L, \"0x%02X\", 128 )", LString("0x80"), LString::fromPrintf ( 1000L, "0x%02X", 128 ) );
  LSTR_TEST ( lclTester, LString::fromPrintf ( 1000, "%d", 123), "123" );
  LSTR_TEST ( lclTester, LString::fromPrintf ( 1000, "%s %s", "Hello", "World" ), "Hello World" );

  LString changer = "----------";
  LSTR_TEST_A ( lclTester, changer, changer.change("-","."), ".........." );
  LSTR_TEST_A ( lclTester, changer, changer.change("..........","1234567890"), "1234567890" );
  LSTR_TEST_A ( lclTester, changer, changer.change("456","---"), "123---7890" );
  LSTR_TEST_A ( lclTester, changer, changer.change("---", "<<--->>"), "123<<--->>7890" );
  LSTR_TEST_A ( lclTester, changer, changer.change("<<--->>","456"), "1234567890" );
  LSTR_TEST_A ( lclTester, changer, changer.change("01234", "XXX"), "1234567890" );
  LSTR_TEST_A ( lclTester, changer, changer.change("89012", "XXX"), "1234567890" );

  LString str_prepend;
  LSTR_TEST_A ( lclTester, str_prepend, str_prepend.prepend ( NULL ), "" );
  LSTR_TEST_A ( lclTester, str_prepend, str_prepend.prepend ( "" ), "" );
  LSTR_TEST_A ( lclTester, str_prepend, str_prepend.prepend ( "456" ), "456" );
  LSTR_TEST_A ( lclTester, str_prepend, str_prepend.prepend ( "123" ), "123456" );
  LSTR_TEST_A ( lclTester, str_prepend, str_prepend.prepend ( "0" ), "0123456" );
  LSTR_TEST_A ( lclTester, str_prepend, str_prepend.append ( "789" ), "0123456789" );

  LHash<LString,LString> hash;

  hash.clear ();
  hash.set ( "TEST1", "--TEST1--" );
  hash.set ( "TEST2", "" );

  testReplacer ( "$(TEST1)", hash, "--TEST1--" );
  testReplacer ( "$(TEST2)", hash, "" );
  testReplacer ( "$(TEST3)", hash, "" );

  testReplacer ( "$(TEST1:-XXX)", hash, "--TEST1--" );
  testReplacer ( "$(TEST2:-XXX)", hash, "XXX" );
  testReplacer ( "$(TEST3:-XXX)", hash, "XXX" );
  testReplacer ( "$(TEST1:+XXX)", hash, "XXX" );
  testReplacer ( "$(TEST2:+XXX)", hash, "" );
  testReplacer ( "$(TEST3:+XXX)", hash, "" );

  testReplacer ( "$(TEST1?-XXX)", hash, "--TEST1--" );
  testReplacer ( "$(TEST2?-XXX)", hash, "" );
  testReplacer ( "$(TEST3?-XXX)", hash, "XXX" );
  testReplacer ( "$(TEST1?+XXX)", hash, "XXX" );
  testReplacer ( "$(TEST2?+XXX)", hash, "XXX" );
  testReplacer ( "$(TEST3?+XXX)", hash, "" );

  testReplacer ( "$(TEST1:2)", hash, "TEST1--" );
  testReplacer ( "$(TEST1:2:5)", hash, "TEST1" );

  testReplacer ( "$(TEST1!TRUE:FALSE)", hash, "TRUE" );
  testReplacer ( "$(TEST2!TRUE:FALSE)", hash, "FALSE" );
  testReplacer ( "$(TEST3!TRUE:FALSE)", hash, "FALSE" );

  testReplacer ( "$(TEST1?TRUE:FALSE)", hash, "TRUE" );
  testReplacer ( "$(TEST2?TRUE:FALSE)", hash, "TRUE" );
  testReplacer ( "$(TEST3?TRUE:FALSE)", hash, "FALSE" );

  // Return length now...

  testReplacer ( "$(#TEST1)", hash, /*--TEST1--*/"9" );
  testReplacer ( "$(#TEST2)", hash, "0" );
  testReplacer ( "$(#TEST3)", hash, "0" );

  testReplacer ( "$(#TEST1:-XXX)", hash, /*"--TEST1--"*/"9" );
  testReplacer ( "$(#TEST2:-XXX)", hash, /*"XXX"*/"3" );
  testReplacer ( "$(#TEST3:-XXX)", hash, /*"XXX"*/"3" );
  testReplacer ( "$(#TEST1:+XXX)", hash, /*"XXX"*/"3" );
  testReplacer ( "$(#TEST2:+XXX)", hash, "0" );
  testReplacer ( "$(#TEST3:+XXX)", hash, "0" );

  testReplacer ( "$(#TEST1?-XXX)", hash, /*"--TEST1--"*/"9" );
  testReplacer ( "$(#TEST2?-XXX)", hash, "0" );
  testReplacer ( "$(#TEST3?-XXX)", hash, /*"XXX"*/"3" );
  testReplacer ( "$(#TEST1?+XXX)", hash, /*"XXX"*/"3" );
  testReplacer ( "$(#TEST2?+XXX)", hash, /*"XXX"*/"3" );
  testReplacer ( "$(#TEST3?+XXX)", hash, "0" );

  testReplacer ( "$(#TEST1:2)", hash, /*"TEST1--"*/"7" );
  testReplacer ( "$(#TEST1:2:5)", hash, /*"TEST1"*/"5" );

  testReplacer ( "$(#TEST1!TRUE:FALSE)", hash, /*"TRUE"*/"4" );
  testReplacer ( "$(#TEST2!TRUE:FALSE)", hash, /*"FALSE"*/"5" );
  testReplacer ( "$(#TEST3!TRUE:FALSE)", hash, /*"FALSE"*/"5" );

  testReplacer ( "$(#TEST1?TRUE:FALSE)", hash, /*"TRUE"*/"4" );
  testReplacer ( "$(#TEST2?TRUE:FALSE)", hash, /*"TRUE"*/"4" );
  testReplacer ( "$(#TEST3?TRUE:FALSE)", hash, /*"FALSE"*/"5" );


  // Testing non-standard

  testReplacer ( "$(TEST1!-XXX)", hash, "--TEST1--" );
  testReplacer ( "$(TEST2!-XXX)", hash, "XXX" );
  testReplacer ( "$(TEST3!-XXX)", hash, "XXX" );
  testReplacer ( "$(TEST1!+XXX)", hash, "XXX" );
  testReplacer ( "$(TEST2!+XXX)", hash, "" );
  testReplacer ( "$(TEST3!+XXX)", hash, "" );

  testReplacer ( "$(TEST1|2)", hash, "TEST1--" );
  testReplacer ( "$(TEST1|2|5)", hash, "TEST1" );

  testReplacer ( "$(TEST1!TRUE!FALSE)", hash, "TRUE" );
  testReplacer ( "$(TEST2!TRUE!FALSE)", hash, "FALSE" );
  testReplacer ( "$(TEST3!TRUE!FALSE)", hash, "FALSE" );

  testReplacer ( "$(TEST1?TRUE!FALSE)", hash, "TRUE" );
  testReplacer ( "$(TEST2?TRUE!FALSE)", hash, "TRUE" );
  testReplacer ( "$(TEST3?TRUE!FALSE)", hash, "FALSE" );

  // Testing illegals

  testReplacer ( "$(TEST1+XXX)", hash, "" );

  // Testing own regex

  testReplacerExp ( LString::C_tag_exp_default, "$(", ")" );
  testReplacerExp ( LString::C_tag_exp_bash, "${", "}" );

  lclTester.dumpResults ();
  return 0;
}
