//#include <config.h>

#include <lcl.h>

//#ifdef HAVE_OCI

#include <ldbsystem.oci8.h>
#include <lcl/lapp.h>
#include <lcl/lpointer.h>

LDBDriver_OCI::LDBDriver_OCI ( LHash<LString,LString> const & vars ) :
  LDBDriver ( vars )
{
  valid = false;

  env = NULL;
  server = NULL;
  session = NULL;
  error = NULL;

  connect ();
}

LDBDriver_OCI::~LDBDriver_OCI ()
{
  disconnect ();
}

LManagedPointer<LDBQuery> LDBDriver_OCI::createQuery ()
{
  return new LDBQuery_OCI ( *this );
}

LString LDBDriver_OCI::getError (  ) const
{
  text errbuff [1024];
  sb4 errcode;

  OCIErrorGet ( error, 1, NULL, &errcode, errbuff, sizeof(errbuff), OCI_HTYPE_ERROR );
  return (char*)errbuff;
}

void LDBDriver_OCI::dumpError ( sword status ) const
{
  LTRACE("LDBDriver_OCI", "rc=" + LString(status) );

  switch ( status ) {
  case OCI_NO_DATA:
    LTRACE ( "SQLErrors", "OCI_NO_DATA" );
    break;
  case OCI_SUCCESS_WITH_INFO:
    LTRACE ( "SQLErrors", "OCI_SUCCESS_WITH_INFO: " + getError () );
    break;
  case OCI_NEED_DATA:
    LTRACE ( "SQLErrors", "OCI_NEED_DATA" );
    break;
  case OCI_INVALID_HANDLE:
    LTRACE ( "SQLErrors", "OCI_INVALID_HANDLE" );
    break;
  case OCI_STILL_EXECUTING:
    LTRACE ( "SQLErrors", "OCI_STILL_EXECUTING" );
    break;
  case OCI_ERROR:
    LTRACE ( "SQLErrors", "OCI_ERROR: " + getError () );
    break;
  default:
    LTRACE ( "SQLErrors", "Unkown Error Code" );
  }
  
}

bool LDBDriver_OCI::connect ()
{

  sword status;

  if ( connected () )
    return false;

  valid = false;

  //cout << getOption("HOST") << getUser() <<  getPassword() << getDatabase() << endl;

  LTRACE ( "LDBDriver_OCI", "OCIEnvCreate" );
  if ( OCIEnvCreate ( &env, OCI_THREADED, NULL, NULL, NULL, NULL, 0, NULL ) != OCI_SUCCESS ) {
    disconnect ();
    return false;
  }

  LString appName = theApp.appName();
  LString user = getUser ();
  LString password = getPassword();
  LString database = getDatabase ();

  /*
  if ( getOption("HOST").length() ) {
    database = getOption("HOST") + ":1521:" + database;
  }
  */

  LTRACE ( "LDBDriver_OCI", "OCIHandleAlloc -> SERVER" );
  OCIHandleAlloc ( env, (dvoid**)&server, OCI_HTYPE_SERVER, 0, NULL );

  LTRACE ( "LDBDriver_OCI", "OCIHandleAlloc -> ERROR" );
  OCIHandleAlloc ( env, (dvoid**)&error, OCI_HTYPE_ERROR, 0, NULL );

  LTRACE ( "LDBDriver_OCI", "OCIServerAttach -> " + database );
  if ( (status = OCIServerAttach ( server, error, (text*)(char*)database, database.length(), OCI_DEFAULT ) ) != OCI_SUCCESS ) {
    dumpError ( status );
    disconnect ();
    return false;
  }

  LTRACE ( "LDBDriver_OCI", "OCIHandleAlloc -> SERVICE" );
  OCIHandleAlloc ( env, (dvoid**)&service, OCI_HTYPE_SVCCTX, 0, NULL );

  LTRACE ( "LDBDriver_OCI", "OCIAttrSet -> SERVICE" );
  if ( ( status = OCIAttrSet ( (dvoid *)service, OCI_HTYPE_SVCCTX, (dvoid *)server, (ub4) 0, OCI_ATTR_SERVER, error ) ) != OCI_SUCCESS ) {
    dumpError ( status );
    disconnect ( );
    return false;
  }

  LTRACE ( "LDBDriver_OCI", "OCIHandleAlloc -> SESSION" );
  OCIHandleAlloc ((dvoid *)env, (dvoid **)&session, OCI_HTYPE_SESSION, 0, NULL );

  LTRACE ( "LDBDriver_OCI", "OCIAttrSet -> SESSION -> user: " + user );
  OCIAttrSet ((dvoid *)session, OCI_HTYPE_SESSION, (dvoid *)user, (ub4)user.length(), OCI_ATTR_USERNAME, error);

  LTRACE ( "LDBDriver_OCI", "OCIAttrSet -> SESSION -> password: " + password );
  OCIAttrSet ((dvoid *)session, OCI_HTYPE_SESSION, (dvoid *)password, (ub4)password.length(), OCI_ATTR_PASSWORD, error);

  LTRACE ( "LDBDriver_OCI", "OCISessionBegin" );
  if ( ( status = OCISessionBegin ( service, error, session, OCI_CRED_RDBMS, OCI_DEFAULT) ) != OCI_SUCCESS )  {
    dumpError ( status );
    disconnect ();
    return false;
  }

  LTRACE ( "LDBDriver_OCI", "OCIAttrSet -> SERVICE" );
  OCIAttrSet (  (dvoid *)service, OCI_HTYPE_SVCCTX, (dvoid *)session, (ub4) 0, OCI_ATTR_SESSION, error);

  valid = true;

  return connected();
}

bool LDBDriver_OCI::disconnect ()
{
  if ( !connected() )
    return false;

  OCISessionEnd ( service, error, session, OCI_DEFAULT );
  OCIServerDetach ( server, error, OCI_DEFAULT );

  if ( error )
    OCIHandleFree ( error, OCI_HTYPE_ERROR );
  if ( session )
    OCIHandleFree ( session, OCI_HTYPE_SESSION );
  if ( server )
    OCIHandleFree ( server, OCI_HTYPE_SERVER );
  if ( env )
    OCIHandleFree ( env, OCI_HTYPE_ENV );

  valid = false;

  return true;
}

bool LDBDriver_OCI::connected () const
{
  return valid;
}

luint32 LDBDriver_OCI::getErrorCode ( OCIError * error )
{
  sb4 errorcode;
  OCIErrorGet (error,
	       1,
	       NULL,
	       &errorcode,
	       /*errorbuf*/NULL,
	       /*sizeof(errorbuf)*/0,
	       OCI_HTYPE_ERROR);
  return errorcode;
}

// ==========>>>>>>>>>>>>>>> QUERY

LDBQuery_OCI::LDBQuery_OCI ( LDBDriver_OCI & drv ) :
  LDBQuery ( drv ),
  driver ( drv ),
  statement ( NULL ),
  has_data ( false )
{
  valid = false;
  stmt_type = 0;
}


LDBQuery_OCI::LDBQuery_OCI ( LDBDriver_OCI & drv, LString const & sql ) :
  LDBQuery ( drv, sql ), 
  driver ( drv ),
  statement ( NULL ),
  has_data ( false )
{
  valid = false;
  stmt_type = 0;
}

LDBQuery_OCI::~LDBQuery_OCI ()
{
  valid = false;
  if ( statement ) {
    OCIHandleFree ( statement, OCI_HTYPE_STMT );
    statement = NULL;   
  }
  if ( num_fields>=0 ) {
    for ( long i = 0; i<num_fields; i++ ) {
      free ( bufferTable[i] );
    }
  }
  num_fields = -1;
}

bool LDBQuery_OCI::prepare ( LString const & sql )
{
  LTRACE ( "SQL", "preparing: '" + sql + "'");

  stmt_type = 0;
  valid = false;

  if ( statement ) {
    LTRACE ( "LDBQuery_OCI", "Freeing old statement handle" );
    OCIHandleFree ( statement, OCI_HTYPE_STMT );
    statement = NULL;
    if ( num_fields ) {
      for ( long i = 0; i<num_fields; i++ ) {
	free ( bufferTable[i] );
      }
    }
  }
  valid = false;
  has_data = false;
  num_fields = -1;

  LTRACE ( "LDBQuery_OCI", "OCIHandleAlloc -> STMT" );
  OCIHandleAlloc ( driver.env, (dvoid**)&statement, OCI_HTYPE_STMT, 0, NULL );

  LTRACE ( "LDBQuery_OCI", "OCIStmtPrepare: sql: " + sql );
  if ( ( status = OCIStmtPrepare ( statement, driver.error, (CONST text*)(char const*)sql, sql.length(), OCI_NTV_SYNTAX, OCI_DEFAULT ) ) != OCI_SUCCESS ) {
    driver.dumpError ( status );
    return false;
  }

  if ( ( status = OCIAttrGet ( (dvoid*)statement, OCI_HTYPE_STMT, (dvoid**)&stmt_type, 0, OCI_ATTR_STMT_TYPE, driver.error ) ) != OCI_SUCCESS ) {
    driver.dumpError ( status );
    return false;
  }

  switch ( stmt_type ) {
  case OCI_STMT_SELECT:
    LTRACE ( "LDBQuery_OCI", "Type -> SELECT" );
    break;
  case OCI_STMT_UPDATE:
    LTRACE ( "LDBQuery_OCI", "Type -> UPDATE" );
    break;
  case OCI_STMT_INSERT:
    LTRACE ( "LDBQuery_OCI", "Type -> INSERT" );
    break;
  case OCI_STMT_DELETE:
    LTRACE ( "LDBQuery_OCI", "Type -> DELETE" );
    break;
  case OCI_STMT_ALTER:
    LTRACE ( "LDBQuery_OCI", "Type -> ALTER" );
    break;
  case OCI_STMT_DROP:
    LTRACE ( "LDBQuery_OCI", "Type -> DROP" );
    break;
  case OCI_STMT_CREATE:
    LTRACE ( "LDBQuery_OCI", "Type -> CREATE" );
    break;
  case OCI_STMT_BEGIN:
    LTRACE ( "LDBQuery_OCI", "Type -> BEGIN" );
    break;
  case OCI_STMT_DECLARE:
    LTRACE ( "LDBQuery_OCI", "Type -> DECLARE" );
    break;
  default:
    LTRACE ( "LDBQuery_OCI", "Unkown statement..." );
    break;
  }

  return true;
}

bool LDBQuery_OCI::exec ()
{
  //  LTRACE ( "SQL", "Executing: '" + sql + "'");

  if ( stmt_type == OCI_STMT_SELECT ) {
    LTRACE ( "LDBQuery_OCI", "SELECT statement, extra processing ... " );

    LTRACE ( "LDBQuery_OCI", "OCIStmtExecute ( OCI_DESCRIBE_ONLY )" );
    status = OCIStmtExecute ( driver.service, statement, driver.error, 1, 1, NULL, NULL, OCI_DESCRIBE_ONLY );
    if ( ( status != OCI_SUCCESS ) && ( status != OCI_SUCCESS_WITH_INFO ) ) {
      driver.dumpError ( status );
      return false;
    }
    
    // find out about the select list ...
    ub4 numcols = 0;
    text * col_name;
    ub4 col_name_len;
    char tmp[4096];
        
    OCIAttrGet ( statement, OCI_HTYPE_STMT, &numcols, 0, OCI_ATTR_PARAM_COUNT, driver.error );
    fieldTable = LTable<LString> ( numcols );
    dataTable = LTable<LString> ( numcols );
    defineTable = LTable<OCIDefine*> ( numcols );
    bufferTable = LTable<char*> ( numcols );
    indTable = LTable<sb2> ( numcols );

    num_fields = numcols;
    for ( luint32 i = 1; i<=numcols; i++ ) {
      OCIParam * param;
      OCIParamGet ( statement, OCI_HTYPE_STMT, driver.error, (dvoid**)&param, i );
      OCIAttrGet ( (dvoid*)param, OCI_DTYPE_PARAM, (dvoid**)&col_name, (ub4*) &col_name_len, OCI_ATTR_NAME, driver.error );
      memcpy ( tmp, col_name, MIN(col_name_len,sizeof(tmp)) );
      //      col_name[col_name_len] = 0;
      tmp[MIN(col_name_len,sizeof(tmp))] = 0;
      LTRACE ( "LDBQuery_OCI", "Col #" + LString(i) + " -> " + LString((char*)tmp/*col_name*/) );
      fieldTable[i-1] = LString ( (char*)tmp/*col_name*/ );

      bufferTable[i-1] = (char*)malloc ( 2048 );
      // SQLT_STR
      if ( ( status = OCIDefineByPos ( statement, &defineTable[i-1], driver.error, (ub4)i, (dvoid*)(char*)bufferTable[i-1], (sb4)2048, SQLT_STR, &indTable[i-1], NULL, NULL, OCI_DEFAULT ) ) != OCI_SUCCESS ) {
	driver.dumpError ( status );
      }
    }
    
  }

  LTRACE ( "LDBQuery_OCI", "OCIStmtExecute" );
  if ( stmt_type == OCI_STMT_SELECT ) {
    status = OCIStmtExecute ( driver.service, statement, driver.error, 0, 0, NULL, NULL, OCI_DEFAULT );
  } else {
    status = OCIStmtExecute ( driver.service, statement, driver.error, 1, 0, NULL, NULL, OCI_COMMIT_ON_SUCCESS );
  }

  switch ( status )
    {

    case OCI_SUCCESS:
      if ( stmt_type == OCI_STMT_SELECT )
	{
	  has_data = true;
	  LTRACE ( "LDBQuery_OCI", "OCI_SUCCESS - with data" );
	}
      else
	{
	  LTRACE ( "LDBQuery_OCI", "OCI_SUCCESS" );
	}
      valid = true;
      break;

    case OCI_NO_DATA:
      LTRACE ( "LDBQuery_OCI", "OCI_NO_DATA" );
      valid = true;
      break;

    default:
      driver.dumpError ( status );
      break;
    }

  return valid;
}

bool LDBQuery_OCI::exec ( LString const & sql )
{

  if ( !prepare ( sql ) )
    return false;
  return exec ( );


}

bool LDBQuery_OCI::hasResult () const
{
  if ( !isValid() )
    return false;

  // FIXME
  // has_data should be ok!

  return has_data;
}

bool LDBQuery_OCI::isValid () const
{
  return ( valid ) && ( statement != NULL );
}

long LDBQuery_OCI::numFields () const
{
  if ( !hasResult() )
    return -1;

  return num_fields;
}

long LDBQuery_OCI::numRows () const
{
  if ( !hasResult () )
    return -1;

  ub4 numrows;

  OCIAttrGet ( statement, OCI_HTYPE_STMT, &numrows, 0, OCI_ATTR_ROW_COUNT, driver.error );

  return numrows;
}

LString LDBQuery_OCI::getField ( long l) const
{
  if ( !hasResult () )
    return "";

  try {
    return dataTable[l];
  }
  catch ( ... ) {
    return "";
  }
}

LString LDBQuery_OCI::getField ( LString const & name ) const
{
  if ( !hasResult () )
    return "";

  long i = getFieldIdx ( name );
  if ( i < 0 )
    return "";

  return dataTable[i];
}

LString LDBQuery_OCI::getFieldName ( long i ) const
{
  if ( !isValid() )
    return "";

  try {
    return fieldTable[i];
  }
  catch ( ... ) {
    return "";
  }
}

long LDBQuery_OCI::getFieldIdx ( LString const & name ) const
{
  try {
    long i = 0;
    while ( true ) {
      if ( fieldTable[i] == name )
	return i;
      i++;
    }
  }
  catch ( ... ) {
    return -1;
  }
}

bool LDBQuery_OCI::fetchNext ( LHash<LString, LString> & hash )
{
  return sys_fetchNext ( hash );
}

bool LDBQuery_OCI::fetchNext ()
{
  return sys_fetchNext ();
}

bool LDBQuery_OCI::sys_fetchNext ( )
{
  if ( !hasResult () ) {
    return false;
  }

  LTRACE ( "LDBDriver_OCI", "OCIStmtFetch" );
  status = OCIStmtFetch ( statement, driver.error, 1, OCI_FETCH_NEXT, OCI_DEFAULT );
  if ( (status != OCI_SUCCESS) &&
       (status != OCI_SUCCESS_WITH_INFO)
       ) {
    driver.dumpError ( status );
    return false;
  }
  for ( long i = 0; i<num_fields; i++ ) {
    LTRACE ( "LDBQuery_OCI", "Inidicator: " + LString(indTable[i]) );
    if ( indTable[i] != -1 )
      dataTable[i] = bufferTable[i];
    else
      dataTable[i] = "";
  }

  return true;
}

bool LDBQuery_OCI::sys_fetchNext ( LHash<LString,LString> & hash )
{
  if ( !hasResult () ) {
    return false;
  }

  LTRACE ( "LDBDriver_OCI", "OCIStmtFetch" );
  status = OCIStmtFetch ( statement, driver.error, 1, OCI_FETCH_NEXT, OCI_DEFAULT );
  if ( (status != OCI_SUCCESS) &&
       (status != OCI_SUCCESS_WITH_INFO)
       ) {
    driver.dumpError ( status );
    return false;
  }
  for ( long i = 0; i<num_fields; i++ ) {
    LTRACE ( "LDBQuery_OCI", "Inidicator: " + LString(indTable[i]) );
    if ( indTable[i] != -1 )
      hash.set ( getFieldName(i), bufferTable[i] );
    else
      hash.set ( getFieldName(i), "" );
  }
  return true;
}


LManagedPointer<LDBBind> LDBQuery_OCI::createBind ( LString const & name )
{
  return new LDBBind_OCI ( *this, name );
}


// ===========>>>>>>>>> Bind

LDBBind_OCI::LDBBind_OCI ( LDBQuery_OCI & _query, LString const & name, unsigned long buffer_len ) :
  query ( _query ),
  bind(NULL),
  buffer(NULL),
  buffer_status ( 0 )
{

  buffer = (char*)malloc ( buffer_len+1 );

  status = OCIBindByName ( query.statement, &bind, query.driver.error, (text*)(const char*)name, name.length(), (ub1*)buffer, buffer_len+1, SQLT_STR, (dvoid*)&buffer_status, (ub2*)0, (ub2*)0, (ub4)0, (ub4*)0, OCI_DEFAULT);
  if ( status != OCI_SUCCESS )
    {
      query.driver.dumpError ( status );
    }
}

LDBBind_OCI::~LDBBind_OCI ( )
{
  free ( buffer );
  
  // Nothing .. will be free by statement...
}

bool LDBBind_OCI::isValid () const
{
  return bind != NULL;
}

LString LDBBind_OCI::getValue () const
{
  return buffer;
}

void LDBBind_OCI::setValue ( LString const & value )
{
  strcpy ( buffer, value );
}

//#endif

LCL_PLUGIN_INIT("OCI", "A LDBSystem OCI PlugIn", 2);
LCL_PLUGIN_FACTORY_WITH_ARGS(LDBDriver_OCI);

