#ifndef _LDBSYSTEM_OCI_H_
#define _LDBSYSTEM_OCI_H_

#include <lcl.h>
#include <lcl/ldbsystem.h>

#include <lcl/lmutex.h>

//#ifdef HAVE_OCI8

#include <oci.h>

class LDBDriver_OCI;
class LDBQuery_OCI;
class LDBBind_OCI;

class LDBDriver_OCI : public LDBDriver {
  LHash<LString,LString> configHash;

  friend class LDBQuery_OCI;
  friend class LDBBind_OCI;

 protected:
  OCIEnv * env;
  OCIServer * server;
  OCIError * error;
  OCISession * session;
  OCISvcCtx * service;

  bool valid;
  LMutex lock;

  void dumpError (sword status ) const;

  virtual LManagedPointer<LDBQuery> createQuery ();

 public:

  LDBDriver_OCI ( LHash<LString,LString> const & vars );
  ~LDBDriver_OCI ();

  //  LDBQuery * query ( LString const & );

  virtual bool connect ();
  virtual bool disconnect ();

  virtual bool connected () const;

  LString getError ( ) const;

  luint32 getErrorCode ( OCIError * error );
};

class LDBQuery_OCI : public LDBQuery {
 friend class LDBBind_OCI;
 protected:

  LDBDriver_OCI & driver;
  OCIStmt * statement;
  sword status;
  bool has_data;
  ub2 stmt_type;

  LTable<LString> fieldTable;
  LTable<LString> dataTable;
  LTable<OCIDefine*> defineTable;
  LTable<char*> bufferTable;
  LTable<sb2> indTable;
  long num_fields;


  bool sys_fetchNext ();
  bool sys_fetchNext ( LHash<LString,LString> & hash );

 public:
  LDBQuery_OCI ( LDBDriver_OCI & );
  LDBQuery_OCI ( LDBDriver_OCI &, LString const & sql );
  virtual ~LDBQuery_OCI ();

  virtual bool prepare ( LString const & );
  virtual bool exec ();
  virtual bool exec ( LString const & );

  //  virtual bool exec ( LHash<LString, LString> const & );

  virtual long numFields () const;
  virtual long numRows () const;

  virtual bool fetchNext ( LHash<LString, LString> & );
  virtual bool fetchNext ();
          
  virtual LString getField ( long ) const;
  virtual LString getField ( LString const & ) const;

  virtual LString getFieldName ( long ) const;
  virtual long getFieldIdx ( LString const & ) const;

  virtual bool hasResult () const;
  virtual bool isValid () const;

  virtual LManagedPointer<LDBBind> createBind ( LString const & name );
};

class LDBBind_OCI : public LDBBind {
  protected:
    LDBQuery_OCI & query;
    OCIBind * bind;
    char * buffer;
    sword status;
    sb2 buffer_status;

  public:
    LDBBind_OCI ( LDBQuery_OCI & db_query, LString const & name, unsigned long len = 1024 );
    virtual ~LDBBind_OCI ();
    bool isValid() const;

    virtual LString getValue () const;
    virtual void setValue ( LString const & );
};

/*#endif*/ //HAVE_OCI

#endif
